import Utils from '../Util/Utils.js';
/**
 * Represents a request object with generic types for input, query parameters, and path parameters.
 * @class Request
 * @template InputType - The type of the input data for the request.
 * @template PathParamsType - The type of the path parameters for the request.
 * @template QueryParamsType - The type of the query parameters for the request.
 */
export default class Request {
    /**
     * Constructs a new instance of the class.
     * @param {APIGatewayEvent} requestEvent - The API Gateway event object.
     * @param {Context} context - The context object.
     * @param {Logger} logger - The logger object.
     * @returns None
     */
    constructor(requestEvent, context, logger) {
        this.requestEvent = requestEvent;
        this.context = context;
        logger.debug('Request info:', requestEvent);
        logger.debug('Request context:', context);
    }
    /**
     * Checks if the specified query parameter exists and has a valid value.
     * @param {keyof QueryParamsType} paramName - The name of the query parameter to check.
     * @returns {boolean} - True if the query parameter exists and has a valid value, false otherwise.
     */
    containsQueryParam(paramName) {
        const val = this.getQueryParam(paramName);
        return !!val && (Utils.isValidString(val) || Utils.isValidNumber(val));
    }
    /**
     * Retrieves the value of a query parameter from the URL.
     * @param {keyof QueryParamsType} paramName - The name of the query parameter to retrieve.
     * @returns {string | null} The value of the query parameter, or null if it does not exist.
     */
    getQueryParam(paramName) {
        return Utils.caseInsensitiveObjectForKey(this.requestEvent.queryStringParameters, String(paramName));
    }
    /**
     * Retrieves the value of the specified header from the request event headers.
     * @param {string} headerName - The name of the header to retrieve.
     * @returns {string | null} - The value of the header, or null if the header is not found.
     */
    getHeader(headerName) {
        return Utils.caseInsensitiveObjectForKey(this.requestEvent.headers, headerName);
    }
    /**
     * Retrieves the value of a context parameter from the request context object.
     * @param {string} cxtParam - The name of the context parameter to retrieve.
     * @returns The value of the context parameter, or null if it does not exist.
     */
    getContextParam(cxtParam) {
        return Utils.caseInsensitiveObjectForKey(this.requestEvent.requestContext, cxtParam);
    }
    /**
     * Checks if the given parameter name exists in the PathParamsType object.
     * @param {keyof PathParamsType} paramName - The name of the parameter to check.
     * @returns {boolean} - True if the parameter exists, false otherwise.
     */
    containsPathParam(paramName) {
        const val = this.getPathParam(paramName);
        return !!val && (Utils.isValidString(val) || Utils.isValidNumber(val));
    }
    /**
     * Retrieves the value of a specific path parameter from the URL.
     * @param {keyof PathParamsType} paramName - The name of the path parameter to retrieve.
     * @returns {string} The value of the path parameter, or null if it does not exist.
     */
    getPathParam(paramName) {
        return Utils.caseInsensitiveObjectForKey(this.requestEvent.pathParameters, String(paramName));
    }
    /**
     * Retrieves the body of the request event and parses it if it is a string.
     * @returns {InputType} The parsed body of the request event.
     */
    getBody(raw) {
        const body = this.requestEvent.body;
        if (raw)
            return Utils.isHybridlessContainer() ? this.requestEvent['rawBody'] : body;
        if (typeof body === 'string' || body instanceof String) {
            try {
                return JSON.parse(body);
            }
            catch (e) {
                console.error('Error while getting request body!', e);
            }
        }
        return body;
    }
    /**
     * Retrieves the path from the request event.
     * @returns {string} The path of the request event.
     */
    getPath() {
        return this.requestEvent.path;
    }
    /**
     * Retrieves the HTTP method of the current request.
     * @returns {string} The HTTP method of the request.
     */
    getMethod() {
        if (this.requestEvent.httpMethod) {
            const httpMethod = this.requestEvent.httpMethod.toUpperCase();
            if (httpMethod in HttpMethod) {
                return HttpMethod[httpMethod];
            }
        }
        throw new Error(`Invalid HTTP method: ${this.requestEvent.httpMethod}`);
    }
    /**
     * Retrieves the path parameters from the request event.
     * @returns {PathParamsType | null} - The path parameters object, or null if not found.
     */
    getPathParams() {
        // type conversion guaranteed by validation
        return this.requestEvent.pathParameters;
    }
    /**
     * Retrieves the query parameters from the request event.
     * @returns {QueryParamsType | null} - The query parameters object, or null if not found.
     */
    getQueryParams() {
        // type conversion guaranteed by validation
        return this.requestEvent.queryStringParameters;
    }
    /**
     * Retrieves the value of the 'Authorization' header from the request.
     * @returns The value of the 'Authorization' header, or null if it is not present.
     */
    getAuthorizationHeader() {
        return this.getHeader('Authorization');
    }
    /**
     * Retrieves the request ID associated with the current execution context.
     * @returns {string} The request ID.
     */
    getRequestID() {
        if (this.context.awsRequestId)
            return this.context.awsRequestId;
        return this.requestEvent.requestContext ? this.requestEvent.requestContext.requestId : 'unknown';
    }
    /**
     * Retrieves the origin IP address of the request.
     * @returns {string} The origin IP address. If the IP address is not available, it returns 'unknown'.
     */
    getOriginIP() {
        var _a;
        const origin = (_a = this.getContextParam('identity')) === null || _a === void 0 ? void 0 : _a.sourceIp;
        const hOrigin = this.getHeader('X-Forwarded-For');
        return origin ? origin : hOrigin ? hOrigin : 'unknown';
    }
    /**
     * Sets the fixed path parameters in the request event object.
     * @param {any[]} keys - An array of keys representing the path parameter names.
     * @param {any[]} result - An array of values representing the path parameter values.
     * @returns None
     */
    setFixedPathParams(keys, result) {
        if (!this.requestEvent.pathParameters)
            this.requestEvent.pathParameters = {};
        keys.forEach((key, index) => {
            var _a;
            if ((_a = this.requestEvent) === null || _a === void 0 ? void 0 : _a.pathParameters) {
                this.requestEvent.pathParameters[key.name] = result[index + 1];
            }
        });
    }
}
/**
 * Enum representing the HTTP methods.
 */
export var HttpMethod;
(function (HttpMethod) {
    HttpMethod["GET"] = "GET";
    HttpMethod["HEAD"] = "HEAD";
    HttpMethod["POST"] = "POST";
    HttpMethod["PUT"] = "PUT";
    HttpMethod["DELETE"] = "DELETE";
    HttpMethod["CONNECT"] = "CONNECT";
    HttpMethod["OPTIONS"] = "OPTIONS";
    HttpMethod["TRACE"] = "TRACE";
    HttpMethod["PATCH"] = "PATCH";
})(HttpMethod || (HttpMethod = {}));
//# sourceMappingURL=Request.js.map