var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import Globals from '../Globals.js';
/**
 * CustomError class that extends the built-in Error class.
 * @class
 * @extends Error
 * @param {any} body - The body of the error, which can be an object or a string.
 * @constructor
 * @property {string} name - The name of the error.
 */
class CustomError extends Error {
    constructor(body) {
        super(body.message || 'Unknown error!');
        this.name = body.error || 'UnknownError';
    }
}
/**
 * Represents a response object with various methods for building and manipulating the response.
 * @template BodyType - The type of the response body.
 */
export default class Response {
    /**
     * Constructs a new Response object with the given status code, body, and optional behavior.
     * @param {number} statusCode - The HTTP status code of the response.
     * @param {BodyType} body - The body of the response.
     * @param {Object} [optBehaviour] - Optional behavior configuration for the response.
     * @param {boolean} [optBehaviour.shouldStream] - Indicates whether the response should be streamed.
     * @param {boolean} [optBehaviour.rawBody] - Indicates whether the response body should be treated as raw data.
     * @param {boolean} [optBehaviour.throwOnErrors] - Indicates whether errors should be thrown for non-successful status codes.
     * @param {boolean} [optBehaviour
     */
    constructor(statusCode, body, optBehaviour) {
        // response
        this.statusCode = statusCode;
        this.body = body;
        this.headers = {
            'Access-Control-Allow-Origin': '*',
            'Access-Control-Allow-Credentials': true,
            'Content-Type': 'application/json',
        };
        // behaviour
        this.isPipingOut = false; //internal -- flag to indicate streaming out has started and avoid double stream call
        // options
        this.shouldStream = !!(optBehaviour === null || optBehaviour === void 0 ? void 0 : optBehaviour.shouldStream); //
        this.rawBody = !!(optBehaviour === null || optBehaviour === void 0 ? void 0 : optBehaviour.rawBody);
        this.throwOnErrors = !!(optBehaviour === null || optBehaviour === void 0 ? void 0 : optBehaviour.throwOnErrors);
        this.disableTransactionID = !!(optBehaviour === null || optBehaviour === void 0 ? void 0 : optBehaviour.disableTransactionID);
    }
    /**
     * Get the status code of the response.
     * @returns {number} The status code.
     */
    getCode() {
        return this.statusCode;
    }
    /**
     * Get the body of the object.
     * @returns {BodyType} The body of the object.
     */
    getBody() {
        return this.body;
    }
    /**
     * Appends a key-value pair into the body object.
     * @param {string} key - The key to append.
     * @param {any} value - The value to append.
     * @returns None
     */
    appendIntoBody(key, value) {
        this.body[key] = value;
    }
    /**
     * Appends a header to the existing headers object.
     * @param {string} key - The key of the header.
     * @param {any} value - The value of the header.
     * @returns None
     */
    appendHeader(key, value) {
        this.headers[key] = value;
    }
    /**
     * Builds the response for the given context and transaction.
     * @param {Context} context - The context object.
     * @param {Transaction<any, any, any>} transaction - The transaction object.
     * @param {boolean} optDoNotCallContext - Optional flag to indicate whether to call the context or not.
     * @returns {Promise<void>} - A promise that resolves when the response is built.
     */
    build(context, transaction, optDoNotCallContext) {
        return __awaiter(this, void 0, void 0, function* () {
            //Stream support
            if (this.isPipingOut)
                return;
            if (this.shouldStream)
                return this.pipe(context);
            //append default fields
            if (transaction.request.getRequestID() && this.body && !this.disableTransactionID)
                this.appendIntoBody('transactionID', transaction.request.getRequestID()); //append transaction ID
            //Raw response support
            if (this.rawBody)
                return this.rawContext(context, transaction);
            //build response
            const b = Object.assign({ statusCode: this.statusCode, headers: this.headers }, (this.body ? { body: JSON.stringify(this.body) } : {}));
            //log response and respond to context
            transaction.logger.debug(b);
            //Check for transaction response proxy
            if (transaction.responseProxy)
                yield transaction.responseProxy(this);
            //Batch does not succeed directly just on upper transaction (which will should be a batch)
            if (!optDoNotCallContext)
                context.succeed(b);
        });
    }
    /**
     * Private method that pipes the response to the given context.
     * @param {Context} context - The context object provided by AWS Lambda.
     * @returns None
     */
    pipe(context) {
        //Check if not streaming
        this.isPipingOut = true;
        //build response
        const b = {
            statusCode: this.statusCode,
            body: this.body,
            headers: this.headers,
        };
        //log response and respond to context
        context.succeed(b);
    }
    /**
     * Private method that handles the raw context of a transaction.
     * @param {Context} context - The context object.
     * @param {Transaction<null, BodyType>} transaction - The transaction object.
     * @returns None
     */
    rawContext(context, transaction) {
        //log response and respond to context
        transaction.logger.debug(this.body);
        if (this.getCode() <= 200 && this.getCode() <= 299)
            context.succeed(this.body);
        else {
            if (!this.throwOnErrors)
                context.fail(new CustomError(this.body));
            else
                throw new CustomError(this.body);
        }
    }
    /**
     * Generates a response object for a missing path parameter error.
     * @param {string} paramName - The name of the missing path parameter.
     * @returns {Response<ResponseErrorType>} - The response object with error details.
     */
    static MissingParamResponse(paramName) {
        console.warn(`Invalid request - Path parameter ${paramName} is missing.`);
        return new Response(400, {
            err: `Invalid request. Path parameter ${paramName} is missing.`,
            errCode: Globals.ErrorCode_MissingParam,
        });
    }
    /**
     * Creates a response object for a missing query parameter error.
     * @param {string} paramName - The name of the missing query parameter.
     * @returns {Response<ResponseErrorType>} - The response object with error details.
     */
    static MissingQueryResponse(paramName) {
        console.warn(`Invalid request - Query parameter ${paramName} is missing.`);
        return new Response(400, {
            err: `Invalid request. Query parameter ${paramName} is missing.`,
            errCode: Globals.ErrorCode_MissingParam,
        });
    }
    /**
     * Creates a BadRequestResponse object with the given parameters.
     * @param {string} [msg] - The error message.
     * @param {string} [errCode] - The error code.
     * @param {any} [optBody] - Optional additional body data.
     * @returns {Response<ResponseErrorType>} - The BadRequestResponse object.
     */
    static BadRequestResponse(msg, errCode, optBody) {
        console.warn(`Bad request - ${msg}`);
        return new Response(400, Object.assign(Object.assign({ err: msg }, (errCode ? { errCode: errCode } : {})), (optBody || {})));
    }
    /**
     * Creates a BadRequestResponse object with rollback option.
     * @param {string} msg - The error message.
     * @param {string} [errCode] - The error code.
     * @param {any} [optBody] - Optional body to include in the response.
     * @returns {Response<ResponseErrorType>} - The BadRequestResponse object.
     */
    static BadRequestResponseWithRollback(msg, errCode, optBody) {
        console.warn(`Bad request - ${msg}`);
        return new Response(400, Object.assign(Object.assign({ err: msg, rollback: true }, (errCode ? { errCode: errCode } : {})), (optBody || {})));
    }
    /**
     * Creates an unauthorized response with the given error message and error code.
     * @param {string} msg - The error message.
     * @param {string} [errCode] - The error code (optional).
     * @returns {Response<ResponseErrorType>} - The unauthorized response.
     */
    static UnauthorizedResponse(msg, errCode) {
        console.warn(`Denying request - ${msg}`);
        return new Response(401, Object.assign({ err: msg }, (errCode ? { errCode: errCode } : {})));
    }
    /**
     * Creates a success response object with the given body.
     * @param {BodyType} body - The body of the response.
     * @returns {Response<BodyType>} - The success response object.
     */
    static SuccessResponse(body) {
        return new Response(200, (body ? body : {}));
    }
    /**
     * Creates a redirect response with the specified URL.
     * @param {string} url - The URL to redirect to.
     * @returns {Response<null>} - The redirect response.
     */
    static RedirectResponse(url) {
        const resp = new Response(302, null);
        resp.appendHeader('Location', url);
        return resp;
    }
    /**
     * Creates a success response with no content.
     * @returns {Response<null>} A response object with a status code of 204 and no content.
     */
    static SuccessNoContentResponse() {
        return new Response(204, null);
    }
    /**
     * Creates a success response object with a streaming body and specified content type.
     * @param {any} stream - The stream object to be used as the response body.
     * @param {string} contentType - The content type of the response.
     * @returns {Response} - The success response object.
     */
    static SuccessStreamResponse(stream, contentType) {
        const resp = new Response(200, stream, {
            shouldStream: true,
        });
        resp.appendHeader('Connection', 'keep-alive');
        if (contentType)
            resp.appendHeader('Content-Type', contentType);
        return resp;
    }
    /**
     * Creates a simple HTTP response with the given body and optional status code.
     * @param {BodyType} body - The body of the response.
     * @param {number} [optionalCode] - The optional status code of the response. Defaults to 200.
     * @returns {Response<BodyType>} - The created response object.
     */
    static SimpleResponse(body, optionalCode) {
        const resp = new Response(optionalCode || 200, body);
        return resp;
    }
    static StepFunctionResponse(body, optionalCode) {
        const resp = new Response(optionalCode || 200, body, {
            rawBody: true,
            throwOnErrors: true,
        });
        return resp;
    }
}
//# sourceMappingURL=Response.js.map