var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import Transaction from './Transaction.js';
import Response from '../API/Response.js';
import Globals from '../Globals.js';
import Utils from '../Util/Utils.js';
/**
 * EventProcessor class that processes events from an SQS queue.
 * @template ResponseInnerType - The type of the inner response object.
 */
export default class EventProcessor {
    /**
     * Constructs a new instance of the class.
     * @param {SQSEvent} event - The event object representing the incoming SQS message.
     * @param {Context} context - The context object representing the AWS Lambda execution context.
     * @param {TransactionConfig} [config] - Optional configuration object for the transaction.
     * @param {boolean} [allowFailure] - Optional flag indicating whether to allow failure for the transaction.
     * @returns None
     */
    constructor(event, context, config, allowFailure) {
        this.event = event;
        this.context = context;
        this.config = config || {};
        this.allowFailure = !!allowFailure;
    }
    /**
     * Processes an event using the provided execution object and returns a response.
     * @param {EventProcessorExecution<ResponseInnerType>} execution - The execution object containing the event to process.
     * @param {boolean} [doNotDecodeMessage] - Optional flag indicating whether to decode the message.
     * @returns {Promise<Response<ResponseErrorType> | null | SQSBatchResponse>} - A promise that resolves to the response object, or null if no response is available.
     * @throws {Error} - Throws an error if the response code is not within the range of 200 to 299 and failure is not allowed.
     */
    processEvent(execution, doNotDecodeMessage) {
        return __awaiter(this, void 0, void 0, function* () {
            const resp = yield this.processRawEvent(execution, !!doNotDecodeMessage);
            if (!this.allowFailure &&
                resp &&
                resp instanceof Response &&
                !(resp.getCode() >= 200 && resp.getCode() < 300))
                throw new Error(JSON.stringify(resp.getBody() || {}));
            else if (resp)
                return resp;
            return null;
        });
    }
    /**
     * Processes a raw event by executing the provided execution function and handling any errors or failures.
     * @param {EventProcessorExecution<ResponseInnerType>} execution - The execution function to process the event.
     * @param {boolean} doNotDecodeMessage - Flag indicating whether to decode the message or not.
     * @returns {Promise<Response<ResponseErrorType> | null | SQSBatchResponse>} - A promise that resolves to a response object, null, or a SQS batch response.
     */
    processRawEvent(execution, doNotDecodeMessage) {
        return __awaiter(this, void 0, void 0, function* () {
            var _a;
            // safe check for empty events?
            if (this.event.Records && this.event.Records.length > 0) {
                // Regular cleanup of tmp disk to avoid tmp overflow on reused lambdas
                if (!((_a = this.config) === null || _a === void 0 ? void 0 : _a.skipCleanTmp))
                    yield Utils.cleanTemporaryFolder();
                // init transaction for all records
                return yield new Transaction(this.event, this.context, Object.assign(Object.assign({}, this.config), { syncReturn: true })).execute((transaction) => __awaiter(this, void 0, void 0, function* () {
                    // Map records with decoded message when required
                    const decodedRecords = this.event.Records.map(eventRecord => doNotDecodeMessage ? eventRecord.body : JSON.parse(eventRecord.body));
                    // for each available event
                    const failureIDs = [];
                    for (const eventRecordIdx in decodedRecords) {
                        const eventRecord = decodedRecords[eventRecordIdx];
                        const message = this.event.Records[eventRecordIdx];
                        // Call execution
                        const resp = yield execution(transaction, eventRecord);
                        // check for failure
                        if (!resp ||
                            (resp instanceof Response && !((resp === null || resp === void 0 ? void 0 : resp.getCode()) >= 200 && (resp === null || resp === void 0 ? void 0 : resp.getCode()) < 300))) {
                            // response with failures or fail hard at first
                            if (this.allowFailure)
                                failureIDs.push(message.messageId);
                            else
                                return resp;
                        }
                    }
                    // not errored and loop ended - succeeded (might have failures)
                    if (this.allowFailure)
                        return {
                            batchItemFailures: failureIDs.map(id => ({ itemIdentifier: id })),
                        };
                    return Response.SuccessResponse(null);
                }));
            }
            else
                return Response.BadRequestResponse(Globals.ErrorResponseNoRecords, Globals.ErrorCode_NoRecords); // no event to be processed?
        });
    }
}
//# sourceMappingURL=EventProcessor.js.map