var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import Request from '../API/Request.js';
import Response from '../API/Response.js';
import { DatabaseManager } from '../Database/DatabaseManager.js';
import Globals from '../Globals.js';
import Logger from '../Logger/Logger.js';
import Publisher from '../Publisher/Publisher.js';
/**
 * Represents a transaction object that handles the execution of a request and manages the response.
 * @template InputType - The type of the input data for the transaction.
 * @template ResponseInnerType - The type of the inner response data for the transaction.
 * @template MiscRespType - The type of miscellaneous response data for the transaction.
 */
export default class Transaction {
    /**
     * Constructs a new instance of the Transaction class.
     * @param {APIGatewayEvent | SQSEvent | DynamoDBStreamEvent} event - The event object passed to the Lambda function.
     * @param {Context} context - The context object passed to the Lambda function.
     * @param {TransactionConfig} [config] - Optional configuration object for the transaction.
     * @returns None
     */
    constructor(event, context, config) {
        /**
         * The instance of the DatabaseManager class used for managing the database.
         */
        this.databaseManager = DatabaseManager.INSTANCE;
        /**
         * An array of database transactions.
         * @type {DatabaseTransaction[]}
         */
        this.transactions = [];
        const transactionId = context.awsRequestId
            ? context.awsRequestId
            : event.requestContext
                ? event.requestContext.requestId
                : 'unknown';
        // transaction ctx
        this.event = event;
        this.context = context;
        // when set, this will be called with the response context right before calling the context suceed/fail - useful for writing the resp for example.
        this.responseProxy = null;
        // transaction flags
        this.syncReturn = !!(config === null || config === void 0 ? void 0 : config.syncReturn);
        this.retrowErrors = !!(config === null || config === void 0 ? void 0 : config.throwOnErrors); /* retrow internal errors */
        // components
        this.logger = new Logger(config === null || config === void 0 ? void 0 : config.logger, transactionId);
        this.request = new Request(this.event, this.context, this.logger);
        this.publisher = new Publisher(config === null || config === void 0 ? void 0 : config.publisher);
    }
    /**
     * Executes a transaction using the provided execution function and returns a promise
     * that resolves to the response or miscellaneous response.
     * @param {TransactionExecution<Transaction<InputType, ResponseInnerType, MiscRespType>, ResponseInnerType, MiscRespType>} executionFunc - The execution function to be executed.
     * @returns {Promise<Response<ResponseInnerType | ResponseErrorType> | MiscRespType>} - A promise that resolves to the response or miscellaneous response.
     */
    execute(executionFunc) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.executeLoggerFlush(() => __awaiter(this, void 0, void 0, function* () {
                yield this.executeDBTransactions(() => __awaiter(this, void 0, void 0, function* () {
                    return yield this.iexecute(executionFunc);
                }));
            }));
            // return raw response if sync return is requested
            if (this.syncReturn)
                return this.response;
            // allow request to async succeed through lambda context
            return null;
        });
    }
    /**
     * Executes a transaction using the provided execution function and handles the response.
     * @param {TransactionExecution<Transaction<InputType, ResponseInnerType, MiscRespType>, ResponseInnerType, MiscRespType>} executionFunc - The function to execute the transaction.
     * @returns {Promise<boolean>} - A promise that resolves to a boolean indicating whether the execution failed or not.
     */
    iexecute(executionFunc) {
        return __awaiter(this, void 0, void 0, function* () {
            let executionFailed = true; //failed til we say no!
            //safe execution handler
            try {
                //Execute
                this.logger.debug('Starting main request code');
                this.response = yield executionFunc(this);
                //Answer client
                if (this.response && this.response instanceof Response) {
                    yield this.response.build(this.context, this, this.syncReturn);
                    executionFailed = !!(this.response.getBody() && this.response.getBody()['rollback']);
                }
                else if (this.syncReturn && this.response) {
                    this.logger.log('Sync return with different response object');
                    this.logger.debug(this.response);
                    executionFailed = false;
                }
                else {
                    this.response = this.getErrorResponse(Globals.ErrorResponseInvalidServerResponse, Globals.ErrorCode_APIError);
                    yield this.response.build(this.context, this, this.syncReturn);
                    this.logger.error('Invalid response object from main request code.');
                }
            }
            catch (e) {
                /*EXECUTION FAIL*/
                this.logger.error('Exception when executing main request code.');
                this.logger.exception(e);
                //retrow?
                if (this.retrowErrors)
                    throw e;
                //envelope exception?
                this.response = this.getErrorResponse(Globals.ErrorResponseUnhandledError, Globals.ErrorCode_APIError);
                yield this.response.build(this.context, this, this.syncReturn);
            }
            return executionFailed;
        });
    }
    /**
     * Retrieves a database transaction based on the provided database configuration.
     * @param {DbConfig<S>} config - The configuration object for the database.
     * @returns {Promise<DatabaseTransactionType<S, DBSchema>>} A promise that resolves to the database transaction.
     */
    getDbTransaction(config) {
        return __awaiter(this, void 0, void 0, function* () {
            const db = this.getDatabase(config);
            const dbTrans = yield db.transaction();
            this.transactions.push(dbTrans);
            return dbTrans;
        });
    }
    /**
     * Retrieves a database instance based on the provided configuration.
     * @param {DbConfig<S>} config - The configuration object specifying the type of database.
     * @returns {DatabaseImplType<S, DBSchema>} A database instance based on the provided configuration.
     */
    getDatabase(config) {
        return this.databaseManager.create(config);
    }
    /**
     * Get the remaining time in milliseconds for the current execution context.
     * @returns {number} The remaining time in milliseconds, or -1 if the time is not available.
     */
    getRemainingTimeInMillis() {
        var _a, _b;
        return ((_b = (_a = this.context) === null || _a === void 0 ? void 0 : _a.getRemainingTimeInMillis) === null || _b === void 0 ? void 0 : _b.call(_a)) || -1;
    }
    /*
     * Executes a series of database transactions in a safe manner.
     * @param {Function} safeExecution - The function that contains the database transactions to be executed.
     * @returns None
     * @throws {Error} - If an exception occurs during the execution of the transactions and `retrowErrors` is true.
     */
    executeDBTransactions(safeExecution) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                // Execute
                const execFailed = yield safeExecution();
                for (const transaction of [...this.transactions].reverse()) {
                    try {
                        yield transaction[execFailed ? 'closeFailure' : 'closeSuccess']();
                    }
                    catch (e) {
                        this.logger.error('Exception when closing DB transactions after success.');
                        this.logger.exception(e);
                    }
                }
            }
            catch (e) {
                /* this part of the code handle exceptions at transaction level,
                   so probably a bug but we still handle such */
                for (const transaction of [...this.transactions].reverse()) {
                    try {
                        yield transaction.closeFailure();
                    }
                    catch (e) {
                        this.logger.error('Exception when closing DB transactions after failure.');
                        this.logger.exception(e);
                    }
                }
                this.logger.error('Exception when executing DB transactions.');
                this.logger.log(e.stack);
                //retrow?
                if (this.retrowErrors)
                    throw e;
            }
        });
    }
    /**
     * Executes a logger flush operation with error handling and logging.
     * @param {Function} safeExecution - The function to execute safely.
     * @returns None
     * @throws {Error} - If `retrowErrors` is true and an error occurs during execution.
     */
    executeLoggerFlush(safeExecution) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield safeExecution();
            }
            catch (e) {
                this.logger.error('Exception when flushing logs.');
                this.logger.exception(e);
                //retrow?
                if (this.retrowErrors)
                    throw e;
            }
            finally {
                this.logger.debug('Transaction ended');
            }
        });
    }
    /**
     * Generates an error response with the provided error message and code.
     * @param {string} error - The error message to include in the response.
     * @param {string} code - The error code associated with the response.
     * @returns {Response<ResponseErrorType>} - The error response object.
     */
    getErrorResponse(error, code) {
        return Response.BadRequestResponseWithRollback(error, code);
    }
}
//# sourceMappingURL=Transaction.js.map