var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import MemCache from 'node-cache';
import DurationParser from 'parse-duration';
import EnvironmentVar, { EnvironmentType } from './EnvironmentVar.js';
// Hold cached values at nodeVM level
// eslint-disable-next-line no-var
/**
 * Creates a new instance of a memory cache store.
 * @returns {MemCache} A new instance of a memory cache store.
 */
let cacheStore = new MemCache();
/**
 * Represents a configuration object with a specified schema and remote prefix.
 * @template T - The type of the configuration schema.
 */
class Configuration {
    /**
     * Constructs a new instance of the class.
     * @param {T} schema - The schema object.
     * @param {string} remotePrefix - The remote prefix string.
     * @returns None
     */
    constructor(schema, remotePrefix) {
        this.schema = schema;
        this.remotePrefix = remotePrefix;
    }
    /**
     * Retrieves the value of a property from the environment variables or cache.
     * @param {keyof OmitByValueType<ExtractLocal<T>, null>} propName - The name of the property to retrieve.
     * @returns The value of the property.
     */
    get(propName) {
        const propString = propName;
        const propSchema = this.schema[propString];
        let v = this.getCachedValue(propString);
        v =
            v ||
                new EnvironmentVar(propSchema.nameOverride || propString, EnvironmentType.Local, !propSchema.required, !propSchema.noPrefix ? this.remotePrefix : '').syncResolve();
        this.cacheValue(propString, v, propSchema.cachingPolicy);
        return v;
    }
    /**
     * Asynchronously retrieves the value of a property from the remote environment.
     * @param {keyof OmitByValueType<ExtractRemote<T>, null>} propName - The name of the property to retrieve.
     * @returns {Promise<any>} - A promise that resolves to the value of the property.
     */
    asyncGet(propName) {
        return __awaiter(this, void 0, void 0, function* () {
            const propString = propName;
            const propSchema = this.schema[propString];
            // check cache
            let v = this.getCachedValue(propString);
            if (v !== undefined)
                return v;
            // check for local override
            if (Configuration.isLocal && process.env[propString] !== undefined) {
                console.log(`Overriding remote variable ${propString} with local value!`);
                const v = process.env[propString];
                this.cacheValue(propString, v, propSchema.cachingPolicy);
                return v;
            }
            // remote
            v = yield new EnvironmentVar(propSchema.nameOverride || propString, propSchema.isSecure ? EnvironmentType.SecureRemote : EnvironmentType.PlainRemote, !propSchema.required, !propSchema.noPrefix ? this.remotePrefix : '').resolve();
            this.cacheValue(propString, v, propSchema.cachingPolicy);
            return v;
        });
    }
    /**
     * Retrieves the cached value associated with the given key from the cache store.
     * @param {string} valueKey - The key of the value to retrieve from the cache.
     * @returns The cached value associated with the given key, or undefined if the key does not exist in the cache.
     */
    getCachedValue(valueKey) {
        return cacheStore.get(valueKey);
    }
    /**
     * Caches a value with the specified key and expiration policy.
     * @param {string} valueKey - The key to associate with the cached value.
     * @param {any} value - The value to be cached.
     * @param {string} [policy='1d'] - The expiration policy for the cached value. Defaults to '1d' (1 day).
     * @returns None
     */
    cacheValue(valueKey, value, policy = '1d') {
        cacheStore.set(valueKey, value, DurationParser(policy, 's') || '');
    }
    /**
     * Resets the cache by creating a new instance of the MemCache class and assigning it to the cacheStore variable.
     * @returns None
     */
    resetCache() {
        cacheStore = new MemCache();
    }
}
/**
 * Indicates if config is local and should attempt to override
 * remote values from local.
 * @type {string}
 */
Configuration.isLocal = process.env.NODE_ENV == 'local';
export default Configuration;
//# sourceMappingURL=Configuration.js.map