var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { SecretsManagerClient, GetSecretValueCommand } from '@aws-sdk/client-secrets-manager';
import { SSMClient, GetParameterCommand } from '@aws-sdk/client-ssm';
// Explicity setting VM level variables in order to persist
// client across different important and instances.
// eslint-disable-next-line no-var
/**
 * Variables to hold instances of SSMClient and SecretsManagerClient, initialized as null.
 * @type {SSMClient | null} ssmClient - Instance of SSMClient or null if not initialized.
 * @type {SecretsManagerClient | null} secretsClient - Instance of SecretsManagerClient or null if not initialized.
 */
let ssmClient = null, secretsClient = null;
/**
 * An enumeration representing different types of environments.
 * @enum {number}
 * @readonly
 * @property {number} PlainRemote - Represents a plain remote environment.
 * @property {number} SecureRemote - Represents a secure remote environment.
 * @property {number} Local - Represents a local environment.
 */
export var EnvironmentType;
(function (EnvironmentType) {
    EnvironmentType[EnvironmentType["PlainRemote"] = 0] = "PlainRemote";
    EnvironmentType[EnvironmentType["SecureRemote"] = 1] = "SecureRemote";
    EnvironmentType[EnvironmentType["Local"] = 2] = "Local";
})(EnvironmentType || (EnvironmentType = {}));
/**
 * Represents an environment variable with methods to resolve its value based on the environment type.
 * @template T - The type of the environment variable value.
 */
export default class EnvironmentVar {
    /**
     * Constructs a new instance of the EnvironmentVar class.
     * @param {string} paramName - The name of the environment variable.
     * @param {EnvironmentType} type - The type of the environment.
     * @param {boolean} [optional] - Indicates whether the environment variable is optional.
     * @param {string} [paramPrefix] - The prefix to be added to the environment variable name.
     */
    constructor(paramName, type, optional = true, paramPrefix) {
        this.paramName = paramName;
        this.type = type;
        this.optional = !!optional;
        this.paramPrefix = paramPrefix;
    }
    /**
     * Resolves the value of the environment variable synchronously.
     * @returns {T} - The resolved value of the environment variable.
     * @throws {Error} - If the environment type is not 'Local'.
     */
    syncResolve() {
        if (this.type == EnvironmentType.Local)
            return this.getLocalValue();
        else {
            throw new Error("EnvironmentVar syncResolve method can only be called for environments of type 'Local'");
        }
    }
    /**
     * Resolves the value based on the environment type.
     * @returns {Promise<any>} - The resolved value.
     */
    resolve() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.type == EnvironmentType.Local)
                return this.getLocalValue();
            else if (this.type == EnvironmentType.SecureRemote)
                return this.getSecureValue();
            else
                return this.getPlainValue();
        });
    }
    /**
     * Retrieves the value of a local environment variable.
     * @returns {T} - The value of the local environment variable.
     * @throws {Error} - If the local environment variable is not defined and is not optional.
     */
    getLocalValue() {
        const value = process.env[this.paramName];
        if (!value && !this.optional) {
            throw new Error(`Local environment ${this.paramName} is not defined, please reconfigure the application and redeploy!`);
        }
        return value;
    }
    /**
     * Retrieves the plain value of a remote environment parameter.
     * @returns {Promise<T>} - A promise that resolves to the plain value of the parameter.
     * @throws {Error} - If the parameter cannot be retrieved and is not optional.
     */
    getPlainValue() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!ssmClient)
                ssmClient = new SSMClient();
            const pName = `${this.paramPrefix}${this.paramName}`;
            try {
                const data = yield ssmClient.send(new GetParameterCommand({ Name: pName }));
                if (data.Parameter && data.Parameter.Value)
                    return data.Parameter.Value;
            }
            catch (error) {
                console.error(error);
            }
            if (!this.optional) {
                throw new Error(`Unable to retrieve plain remote env value: ${pName}`);
            }
        });
    }
    /**
     * Retrieves a secure value from the secrets manager.
     * @returns {Promise<T>} - A promise that resolves to the secure value.
     * @throws {Error} - Throws an error if the secure value cannot be retrieved and is not optional.
     */
    getSecureValue() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!secretsClient)
                secretsClient = new SecretsManagerClient();
            const pName = `${this.paramPrefix}${this.paramName}`;
            try {
                const data = yield secretsClient.send(new GetSecretValueCommand({ SecretId: pName }));
                if (data.SecretString)
                    return data.SecretString;
            }
            catch (error) {
                console.error(error);
            }
            if (!this.optional) {
                throw new Error(`Unable to retrieve secure remote env: ${pName}`);
            }
        });
    }
}
//# sourceMappingURL=EnvironmentVar.js.map