var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { KMSClient, EncryptCommand, DecryptCommand, } from '@aws-sdk/client-kms';
/**
 * Represents a Crypto class that provides encryption and decryption functionality using AWS KMS.
 */
export default class Crypto {
    /**
     * Constructs a new instance of a KeyManager with the provided region, keyId, and optional encryption algorithm.
     * @param {string} region - The region where the key is stored.
     * @param {string} keyId - The ID of the key.
     * @param {EncryptionAlgorithmSpec} [optEncryptionAlgorithm='RSAES_OAEP_SHA_256'] - The encryption algorithm to use (default is RSAES_OAEP_SHA_256).
     * @returns None
     */
    constructor(region, keyId, optEncryptionAlgorithm) {
        this.region = region;
        this.keyId = keyId;
        this.encryptionAlgorithm = optEncryptionAlgorithm || 'RSAES_OAEP_SHA_256';
        this.client = new KMSClient({ region: this.region });
    }
    /**
     * Encrypts the given data using RSAES_OAEP_SHA_256 encryption algorithm.
     * @param {string | any} data - The data to be encrypted.
     * @returns {Promise<string | null>} - A promise that resolves to the encrypted data as a hexadecimal string.
     */
    encryptData(data) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const resp = yield this.client.send(new EncryptCommand({
                    KeyId: this.keyId,
                    EncryptionAlgorithm: this.encryptionAlgorithm,
                    Plaintext: new TextEncoder().encode(typeof data === 'string' ? data : JSON.stringify(data)),
                }));
                return Buffer.from(resp.CiphertextBlob, 'utf8').toString('hex');
            }
            catch (e) {
                console.error('Encryption failure', e);
                return null;
            }
        });
    }
    /**
     * Decrypts the given data using the RSAES_OAEP_SHA_256 encryption algorithm.
     * @param {string} data - The encrypted data to decrypt.
     * @returns {Promise<string | null>} - A promise that resolves to the decrypted plaintext string.
     * If decryption fails, null is returned.
     */
    decryptData(data) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const resp = yield this.client.send(new DecryptCommand({
                    KeyId: this.keyId,
                    CiphertextBlob: Buffer.from(data, 'hex'),
                    EncryptionAlgorithm: this.encryptionAlgorithm,
                }));
                return new TextDecoder().decode(resp.Plaintext);
            }
            catch (e) {
                console.error('Decryption failure', e);
                return null;
            }
        });
    }
}
//# sourceMappingURL=Crypto.js.map