var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * Abstract class representing a database transaction.
 * Warning: These need to be arrow functions!
 * @class DatabaseTransaction
 */
export class DatabaseTransaction extends Function {
    /**
     * Constructor for a class that interacts with a database using a writer and reader.
     * @param {any} writer - The object responsible for writing to the database.
     * @param {Database<any>} database - The database to interact with.
     * @param {any} [reader] - The object responsible for reading from the database (optional).
     * @returns None
     */
    constructor(writer, database, reader) {
        super('...args', 'return this.transaction(...args)');
        /**
         * Check if the object is open.
         * @returns {boolean} - true if the object is open, false otherwise.
         */
        this.isOpen = () => {
            return this._isOpen;
        };
        /**
         * Begins a transaction asynchronously.
         * @returns {Promise<void>} A Promise that resolves when the transaction has begun.
         * @throws {Error} If the transaction is already open.
         */
        this.begin = () => __awaiter(this, void 0, void 0, function* () {
            if (this._isOpen) {
                throw new Error('Cannot begin, transaction is already opened!');
            }
            yield this.doBegin();
            this._isOpen = true;
        });
        /**
         * Asynchronously commits the transaction.
         * If the transaction is already closed, an error is thrown.
         * @returns Promise<void>
         * @throws Error if the transaction is already closed
         */
        this.commit = () => __awaiter(this, void 0, void 0, function* () {
            if (!this._isOpen) {
                throw new Error('Cannot commit, transaction is already closed!');
            }
            yield this.doCommit();
            this._isOpen = false;
        });
        /**
         * Rollback the transaction by reverting any changes made within the transaction.
         * If the transaction is already closed, an error is thrown.
         * @returns Promise<void>
         * @throws Error if the transaction is already closed.
         */
        this.rollback = () => __awaiter(this, void 0, void 0, function* () {
            if (!this._isOpen) {
                throw new Error('Cannot rollback, transaction is already closed!');
            }
            yield this.doRollback();
            this._isOpen = false;
        });
        /**
         * Closes the success modal, committing or rolling back changes based on the autoCommit setting.
         * If the modal is open and autoCommit is enabled, it will commit the changes.
         * If autoCommit is disabled, it will rollback the changes.
         * @returns {Promise<void>} A promise that resolves once the commit or rollback operation is completed.
         */
        this.closeSuccess = () => __awaiter(this, void 0, void 0, function* () {
            if (this._isOpen) {
                if (this.database.config['autoCommit']) {
                    yield this.doCommit();
                }
                else {
                    yield this.doRollback();
                }
            }
        });
        /**
         * Closes the failure by performing a rollback if the failure is currently open.
         * @returns {Promise<void>} A promise that resolves once the rollback is completed.
         */
        this.closeFailure = () => __awaiter(this, void 0, void 0, function* () {
            if (this._isOpen) {
                yield this.doRollback();
            }
        });
        this.writer = writer;
        this.database = database;
        this.reader = reader;
    }
    /**
     * Creates a proxy instance for the given subclass of DatabaseTransaction.
     * The proxy handles method binding and transaction execution based on the subclass state.
     * @param {T} subclass - The subclass of DatabaseTransaction to proxy.
     * @returns A proxied instance of the subclass with method binding and transaction execution logic.
     */
    static proxyInstance(subclass) {
        const bind = (target, key) => typeof target[key] === 'function' ? target[key].bind(target) : target[key];
        return new Proxy(subclass, {
            get(target, p) {
                if (target[p] !== undefined) {
                    return bind(target, p);
                }
                else if (target._isOpen) {
                    return bind(target.transaction, p);
                }
                return undefined;
            },
            apply(target, thisArg, argArray) {
                if (target._isOpen) {
                    return target.transaction(...argArray);
                }
                throw new Error('Transaction is closed!');
            },
        });
    }
}
//# sourceMappingURL=DatabaseTransaction.js.map