var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { CamelCasePlugin, Kysely, PostgresDialect } from 'kysely';
import pg from 'pg';
import { KyselyTransactionImpl } from './KyselyTransaction.js';
import { Database } from '../../Database.js';
/**
 * Represents a database connection using the Kysely library with support for transactions.
 * @template DBSchema - The schema type for the database.
 */
export class KyselyDatabase extends Database {
    /**
     * Constructor for creating a database connection using the provided configuration.
     * @param {DbConfig<'kysely'>} config - The configuration object for the database connection.
     * @returns None
     */
    constructor(config) {
        super(config);
        this.pgClient = this.dialectFactory(config);
        this.client = this.providerFactory(this.pgClient);
        if (config.readReplica) {
            this.pgReadClient = this.dialectFactory(config.readReplica);
            this.readClient = this.providerFactory(this.pgReadClient);
        }
    }
    /**
     * Override method that starts a new transaction using the provided client and read client.
     * @returns A Promise that resolves to a KyselyTransaction object for the specified database schema.
     */
    transaction() {
        return __awaiter(this, void 0, void 0, function* () {
            return KyselyTransactionImpl.newTransaction(this.client, this, this.readClient);
        });
    }
    /**
     * Creates a database dialect based on the provided configuration.
     * @param {DbBaseConfig} config - The configuration object for the database connection.
     * @returns A database dialect instance based on the provided configuration.
     */
    dialectFactory(config) {
        return new KyselyDatabase.kyselyPgProvider({
            pool: new KyselyDatabase.pgProvider({
                host: config.host,
                port: config.port,
                user: config.username,
                password: config.password,
                database: config.database,
                max: config.maxConnections,
            }),
        });
    }
    /**
     * Creates a provider for interacting with a database using the specified Postgres dialect.
     * @param {PostgresDialect} dialect - The Postgres dialect to use for the database connection.
     * @returns A database provider for interacting with the database.
     */
    providerFactory(dialect) {
        return new KyselyDatabase.kyselyProvider({
            dialect,
            plugins: [new CamelCasePlugin()],
        });
    }
}
/**
 * Represents a PostgreSQL provider for querying data using the PostgresDialect.
 * @type {PostgresDialect}
 */
KyselyDatabase.kyselyPgProvider = PostgresDialect;
/**
 * A public static property that provides access to the Kysely class.
 * This property can be accessed without creating an instance of the class.
 */
KyselyDatabase.kyselyProvider = Kysely;
/**
 * A static property that represents a connection pool for PostgreSQL database connections.
 * @type {Pool}
 */
KyselyDatabase.pgProvider = pg.Pool;
//# sourceMappingURL=KyselyDatabase.js.map