var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { DatabaseTransaction } from '../../DatabaseTransaction.js';
/**
 * Represents a transaction in a Postgres database.
 */
export class KyselyTransactionImpl extends DatabaseTransaction {
    /**
     * Constructs a new instance of the class.
     * @param {Kysely<DBSchema>} delegate - The delegate object for the database query.
     * @param {KyselyDatabase<DBSchema>} database - The database object for the query.
     * @param {Kysely<DBSchema>} [reader] - An optional reader object for the query.
     * @returns None
     */
    constructor(delegate, database, reader) {
        super(delegate, database, reader);
        /**
         * Asynchronously begins a transaction using a writer and resolves the transaction once it is executed.
         * @returns A Promise that resolves with the transaction object once the transaction is executed.
         */
        this.doBegin = () => __awaiter(this, void 0, void 0, function* () {
            this.txWrapper = new Deferred();
            return new Promise(resolve => {
                this.writer
                    .transaction()
                    .execute((trx) => __awaiter(this, void 0, void 0, function* () {
                    this.transaction = trx;
                    resolve(trx); // resolve with tx
                    return this.txWrapper.promise; // wait for wrapper to be solved
                }))
                    .catch(() => {
                    // Don't do anything here. Just swallow the exception.
                });
            });
        });
        /**
         * Executes the commit operation by resolving the transaction wrapper and returning a resolved Promise.
         * @returns A Promise that resolves to null.
         */
        this.doCommit = () => {
            this.txWrapper.resolve(null);
            return Promise.resolve();
        };
        /**
         * Performs a rollback operation by rejecting the transaction wrapper with an error.
         * @returns A resolved Promise after the rollback operation is completed.
         */
        this.doRollback = () => {
            this.txWrapper.reject(new Error('Rollback'));
            return Promise.resolve();
        };
    }
    /**
     * Creates a new database transaction for the given database schema.
     * @param {Kysely<DBSchema>} connection - The connection object for the transaction.
     * @param {KyselyDatabase<DBSchema>} database - The database object for the transaction.
     * @param {Kysely<DBSchema>} [reader] - An optional reader object for the transaction.
     * @returns {Promise<KyselyTransaction<DBSchema>>} A promise that resolves to the created transaction.
     */
    static newTransaction(connection, database, reader) {
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new KyselyTransactionImpl(connection, database, reader);
            yield tx.begin();
            return DatabaseTransaction.proxyInstance(tx);
        });
    }
}
/**
 * Represents a deferred promise that can be resolved or rejected at a later time.
 * @template T - The type of the value that the promise will resolve to.
 */
class Deferred {
    /**
     * Constructor for creating a new Promise instance with resolve and reject functions.
     * @constructor
     */
    constructor() {
        this._promise = new Promise((resolve, reject) => {
            this._reject = reject;
            this._resolve = resolve;
        });
    }
    /**
     * Getter method to retrieve the Promise object.
     * @returns {Promise<T>} A Promise object of type T.
     */
    get promise() {
        return this._promise;
    }
    /**
     * Resolves the Promise with the given value.
     * @param {T | PromiseLike<T>} value - The value to resolve the Promise with.
     * @returns void
     */
    resolve(value) {
        if (this._resolve) {
            this._resolve(value);
        }
    }
    /**
     * Rejects the Promise with the given reason, if the reject function is available.
     * @param {any} reason - The reason for rejecting the Promise.
     * @returns void
     */
    reject(reason) {
        if (this._reject) {
            this._reject(reason);
        }
    }
}
//# sourceMappingURL=KyselyTransaction.js.map