var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { DatabaseTransaction } from '../../DatabaseTransaction.js';
/**
 * Represents a transaction in a Postgres database.
 */
export class PostgresTransactionImpl extends DatabaseTransaction {
    /**
     * Constructs a new instance of the class with the provided writer, database, and optional reader.
     * @param {Pool} writer - The writer pool for database operations.
     * @param {PostgresDatabase} database - The Postgres database instance.
     * @param {Pool} [reader] - The reader pool for database operations (optional).
     */
    constructor(writer, database, reader) {
        super(writer, database, reader);
        /**
         * Initiates a transaction by connecting to the writer and executing a 'BEGIN' query.
         * @returns {Promise<void>} A promise that resolves when the transaction is successfully initiated.
         */
        this.doBegin = () => __awaiter(this, void 0, void 0, function* () {
            this.transaction = yield this.writer.connect();
            return this.transaction.query('BEGIN');
        });
        /**
         * Executes a COMMIT query to commit the current transaction.
         * @returns A Promise that resolves when the COMMIT query is successfully executed.
         */
        this.doCommit = () => {
            return this.transaction.query('COMMIT');
        };
        /**
         * Rolls back the current transaction by executing a 'ROLLBACK' query.
         * @returns A Promise that resolves when the rollback is successful.
         */
        this.doRollback = () => {
            return this.transaction.query('ROLLBACK');
        };
    }
    /**
     * Creates a new database transaction using the provided writer and database instances.
     * @param {Pool} writer - The writer instance for the transaction.
     * @param {PostgresDatabase} database - The database instance for the transaction.
     * @param {Pool} [reader] - Optional reader instance for the transaction.
     * @returns {Promise<PostgresTransaction>} A promise that resolves to a new PostgresTransaction instance.
     */
    static newTransaction(writer, database, reader) {
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new PostgresTransactionImpl(writer, database, reader);
            yield tx.begin(); // defaults to opened
            return DatabaseTransaction.proxyInstance(tx);
        });
    }
}
//# sourceMappingURL=PostgresTransaction.js.map