import abind from 'abind';
import stringify from 'json-stringify-safe';
import stackTrace from 'stack-trace';
import Utils from '../Util/Utils.js';
/**
 * Enumeration of log levels.
 * @enum {string}
 * @property {string} DEBUG - Debug log level.
 * @property {string} INFO - Info log level.
 * @property {string} WARN - Warning log level.
 * @property {string} ERROR - Error log level.
 */
export var LOG_LEVELS;
(function (LOG_LEVELS) {
    LOG_LEVELS["DEBUG"] = "DEBUG";
    LOG_LEVELS["INFO"] = "INFO";
    LOG_LEVELS["WARN"] = "WARN";
    LOG_LEVELS["ERROR"] = "ERROR";
})(LOG_LEVELS || (LOG_LEVELS = {}));
/**
 * A constant that represents the console object to be used for logging.
 * If the console object has a property named 'notGlobalLogger', it is assumed
 * to be a custom logger and the 'origin' property is used as the console object.
 * Otherwise, the global console object is used.
 * @type {Console}
 */
const PURE_CONSOLE = console['notGlobalLogger'] ? console['origin'] : console;
/**
 * The default log function that is used for logging messages.
 * @type {Function}
 */
const DEFAULT_LOG_FUNCTION = PURE_CONSOLE.log.bind(PURE_CONSOLE);
/**
 * Creates a blacklist array by mapping each string in the given array to its lowercase form.
 * The resulting blacklist array is used to filter out sensitive information.
 * @type {string[]} blacklist - An array of strings to be converted to lowercase and used as a blacklist.
 * @returns {string[]} - An array of lowercase strings representing the blacklist.
 */
const blacklist = ['password', 'token', 'accounts'].map(s => s.toLowerCase());
/**
 * Logger class for logging messages with different log levels.
 */
export default class Logger {
    /**
     * Constructs a Logger object with the given configuration and transaction ID.
     * @param {config} config - The configuration object for the logger. Can be undefined.
     * @param {string} transactionID - The ID of the transaction associated with the logger.
     * @returns None
     */
    constructor(config, transactionID) {
        abind(this);
        //
        this.origin = PURE_CONSOLE;
        this._LOG_LEVEL = (config === null || config === void 0 ? void 0 : config.logLevel)
            ? LOG_LEVELS[config === null || config === void 0 ? void 0 : config.logLevel] || LOG_LEVELS.DEBUG
            : LOG_LEVELS.DEBUG;
        this.config = config || {};
        this.transactionID = transactionID;
        this.filterBlacklist = this.config.sensitiveFilteringKeywords
            ? Array.isArray(this.config.sensitiveFilteringKeywords)
                ? this.config.sensitiveFilteringKeywords
                : blacklist
            : [];
        //
        this.setupBindings();
        //
        this.log('Using logger with level: ' + this._LOG_LEVEL.toString());
        this.debug('logger config: ', this.config);
    }
    /**
     * Returns a boolean value indicating whether the notGlobalLogger function is executed successfully.
     * @returns {boolean} - true if the function is executed successfully, false otherwise.
     */
    notGlobalLogger() {
        return true;
    }
    /**
     * Logs the given arguments with the debug log level.
     * @param {...any} args - The arguments to be logged.
     * @returns None
     */
    debug(...args) {
        this.processLog(LOG_LEVELS.DEBUG, args);
    }
    /**
     * Logs the given arguments with the INFO log level.
     * @param {...any} args - The arguments to be logged.
     * @returns None
     */
    log(...args) {
        this.processLog(LOG_LEVELS.INFO, args);
    }
    /**
     * Logs an informational message.
     * @param {...any} args - The message(s) to log.
     * @returns None
     */
    info(...args) {
        this.processLog(LOG_LEVELS.INFO, args);
    }
    /**
     * Logs a warning message with the provided arguments.
     * @param {...any} args - The arguments to be logged as a warning message.
     * @returns None
     */
    warning(...args) {
        this.processLog(LOG_LEVELS.WARN, args);
    }
    /**
     * Logs a warning message to the console.
     * @param {...any} args - The arguments to be logged.
     * @returns None
     */
    warn(...args) {
        this.processLog(LOG_LEVELS.WARN, args);
    }
    /**
     * Logs an error message with the given arguments.
     * @param {...any} args - The arguments to log as an error message.
     * @returns None
     */
    error(...args) {
        this.processLog(LOG_LEVELS.ERROR, args);
    }
    /**
     * Logs an exception with optional additional arguments.
     * @param {any} exception - The exception to log.
     * @param {...any} args - Additional arguments to include in the log.
     * @returns None
     */
    exception(exception, ...args) {
        this.iexception(exception, args);
    }
    /**
     * Sets up the console bindings for logging purposes.
     * @private
     * @returns None
     */
    setupBindings() {
        global.console = {
            debug: (...args) => this.processLog(LOG_LEVELS.DEBUG, args),
            log: (...args) => this.processLog(LOG_LEVELS.INFO, args),
            info: (...args) => this.processLog(LOG_LEVELS.INFO, args),
            warn: (...args) => this.processLog(LOG_LEVELS.WARN, args),
            error: (...args) => this.processLog(LOG_LEVELS.ERROR, args),
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            warning: (...args) => this.processLog(LOG_LEVELS.WARN, args),
            exception: (exception, ...args) => this.iexception(exception, args),
        };
    }
    /**
     * Formats a log message with the specified log level, message, and caller.
     * @param {LOG_LEVELS} level - The log level of the message.
     * @param {Array<string>} msg - An array of strings representing the message.
     * @param {string} caller - The name of the caller function.
     * @returns {string} - The formatted log message.
     */
    formattedLog(level, msg, caller) {
        if (Utils.isHybridlessContainer() && this.transactionID) {
            return `${this.transactionID}` + ` [${level.toString()}] [${caller}] ${msg.join(' ')}`;
        }
        else {
            return `[${level.toString()}] [${caller}] ${msg.join(' ')}`;
        }
    }
    /**
     * Retrieves the name of the caller function at the specified index in the stack trace.
     * @param {number} index - The index of the caller function in the stack trace.
     * @returns {string} The name of the caller function along with the file path and line number.
     */
    callerName(index) {
        var _a, _b, _c, _d, _e, _f;
        const safeIndex = Math.min(index, stackTrace.get().length);
        if (stackTrace.get()[safeIndex]) {
            let callerName = (_c = (_b = (_a = stackTrace === null || stackTrace === void 0 ? void 0 : stackTrace.get()) === null || _a === void 0 ? void 0 : _a[safeIndex]) === null || _b === void 0 ? void 0 : _b.getFileName()) === null || _c === void 0 ? void 0 : _c.split('/');
            callerName = (_d = callerName === null || callerName === void 0 ? void 0 : callerName.slice(callerName === null || callerName === void 0 ? void 0 : callerName.indexOf('src'))) === null || _d === void 0 ? void 0 : _d.join('/');
            return callerName + ':' + ((_f = (_e = stackTrace === null || stackTrace === void 0 ? void 0 : stackTrace.get()) === null || _e === void 0 ? void 0 : _e[safeIndex]) === null || _f === void 0 ? void 0 : _f.getLineNumber());
        }
        return '';
    }
    /**
     * Processes log messages based on the specified log level.
     * @param {LOG_LEVELS} level - The level of the log message.
     * @param {any} args - The arguments to be logged.
     * @returns None
     */
    processLog(level, args) {
        if (level < this._LOG_LEVEL)
            return;
        //get args
        const msg = [];
        for (const arg of args) {
            // Deep clone object so we dont modify source
            const fMsg = this.formatArgument(arg);
            msg.push(fMsg);
        }
        //push into logs stack
        // todo: improve error stack
        this.pushLog(level, this.formattedLog(level, msg, this.callerName(3)));
    }
    formatArgument(arg) {
        if (arg instanceof Error) {
            return arg.message + '\n' + arg.stack;
        }
        if (arg && typeof arg === 'object') {
            return stringify(this.suppressSensitiveInfo(JSON.parse(stringify(arg))), null, 2);
        }
        return `${this.suppressSensitiveInfo(arg)}`;
    }
    /**
     * Logs an exception along with additional arguments and the stack trace.
     * @param {Error} exception - The exception object to log.
     * @param {...any} args - Additional arguments to include in the log.
     * @returns None
     */
    iexception(exception, ...args) {
        //format message
        const msg = [];
        //push exeception
        msg.push(exception.toString() + ' -');
        //get args
        for (const arg of args)
            if (arg != exception)
                msg.push(arg);
        if (exception.stack)
            msg.push(exception.stack); //push Exeception stack at the end
        //push into logs stack
        this.pushLog(LOG_LEVELS.ERROR, this.formattedLog(LOG_LEVELS.ERROR, msg, this.callerName(3)));
    }
    /**
     * Pushes a log message to the console with the specified log level.
     * @param {LOG_LEVELS} level - The log level of the message.
     * @param {string} fMsg - The formatted log message.
     * @returns None
     */
    pushLog(level, fMsg) {
        DEFAULT_LOG_FUNCTION.apply(PURE_CONSOLE, [fMsg]);
    }
    /**
     * Suppresses sensitive information in the given value based on the filter blacklist.
     * @param {any} value - The value to suppress sensitive information from.
     * @returns {string} - The value with sensitive information suppressed.
     */
    suppressSensitiveInfo(value) {
        if (!value || !this.filterBlacklist.length)
            return value;
        const parent = [value];
        const stack = [{ value, parent, key: 0 }];
        while (stack.length > 0) {
            this.suppressSensitiveInfoItem(stack.pop(), stack.push.bind(stack));
        }
        return parent[0];
    }
    suppressSensitiveInfoItem({ value, parent, key }, push) {
        if (!value)
            return;
        if (typeof value === 'string') {
            this.suppressSensitiveString({ value, parent, key }, push);
        }
        else if (Array.isArray(value)) {
            value.forEach((v, index) => push({ value: v, parent: value, key: index }));
        }
        else if (typeof value === 'object') {
            this.suppressSensitiveObject(value, push);
        }
    }
    suppressSensitiveString({ value, parent, key }, push) {
        var _a;
        let modifiedValue = value;
        try {
            // Try to parse json string
            modifiedValue = JSON.parse(value);
            push({ value: modifiedValue, parent, key });
        }
        catch (_b) {
            const lower = value.toLowerCase();
            if (this.filterBlacklist.some(f => lower == f))
                modifiedValue = `**SUPPRESSED_SENSITIVE_DATA** (${((_a = String(modifiedValue)) === null || _a === void 0 ? void 0 : _a.length) || 0} len)`;
        }
        parent[key] = modifiedValue;
    }
    suppressSensitiveObject(value, push) {
        Object.entries(value).forEach(([k, v]) => {
            var _a;
            const lower = k.toLowerCase();
            const match = v && this.filterBlacklist.some(f => lower == f);
            if (match) {
                value[k] = `**SUPPRESSED_SENSITIVE_DATA** (${((_a = String(v)) === null || _a === void 0 ? void 0 : _a.length) || 0} len)`;
            }
            else {
                push({ value: v, parent: value, key: k });
            }
        });
    }
}
//# sourceMappingURL=Logger.js.map