/**
 * Represents a collection of routes that can be added and resolved.
 */
class Routes {
    /**
     * Initializes a new instance of the class.
     * @constructor
     */
    constructor() {
        this.next = {};
    }
    /**
     * Adds a route to the routing system.
     * @param {Route} route - The route to add.
     * @param {string[]} parts - The parts of the route path.
     * @returns None
     * @throws {Error} If a duplicate route is detected.
     */
    addRoute(route, parts) {
        const first = parts.shift();
        if (first) {
            if (first.startsWith(':')) {
                this.variable = this.variable || new Routes();
                this.variable.addRoute(route, parts);
            }
            else {
                this.next[first] = this.next[first] || new Routes();
                this.next[first].addRoute(route, parts);
            }
        }
        else {
            if (this.current)
                throw new Error(`Duplicate route: ${route.method}: ${route.path}`);
            this.current = route;
        }
    }
    /**
     * Resolves a route based on the given parts.
     * @param {string[]} parts - An array of route parts.
     * @returns {Route} The resolved route.
     */
    resolveRoute(parts) {
        var _a;
        const first = parts.shift();
        if (first) {
            if (first in this.next) {
                return this.next[first].resolveRoute(parts);
            }
            return (_a = this.variable) === null || _a === void 0 ? void 0 : _a.resolveRoute(parts);
        }
        return this.current;
    }
}
/**
 * A class that resolves routes based on the provided configuration.
 * @class RouteResolver
 */
export default class RouteResolver {
    /**
     * Constructs a new instance of the Router class with the given configuration.
     * @param {RouterConfig} config - The configuration object for the router.
     * @returns None
     */
    constructor(config) {
        this.routes = {};
        this.buildRoutes(config);
    }
    /**
     * Resolves a route based on the given HTTP method and path.
     * @param {string} method - The HTTP method of the request.
     * @param {string} path - The path of the request.
     * @returns {Route | undefined} - The resolved route or undefined if no route is found.
     */
    resolveRoute(method, path) {
        var _a;
        const parts = path.split('/').filter(p => p.length);
        return (_a = this.routes[method]) === null || _a === void 0 ? void 0 : _a.resolveRoute(parts);
    }
    /**
     * Builds the routes for the router based on the given configuration.
     * @param {RouterConfig} config - The router configuration object.
     * @returns None
     */
    buildRoutes(config) {
        const addRoute = (path, route) => {
            const parts = path.split('/').filter(p => p.length);
            this.routes[route.method] = this.routes[route.method] || new Routes();
            this.routes[route.method].addRoute(route, parts);
        };
        for (const route of config.routes) {
            if (Array.isArray(route.path)) {
                for (const path of route.path)
                    addRoute(path, route);
            }
            else
                addRoute(route.path, route);
        }
    }
}
//# sourceMappingURL=RouteResolver.js.map