var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { pathToRegexp } from 'path-to-regexp';
import Response from '../../API/Response.js';
import Transaction from '../../BaseEvent/Transaction.js';
import Validator from '../../Validation/Validator.js';
import RouteResolver from '../RouteResolver.js';
/**
 * Represents a server that handles serverless events and routes them to appropriate handlers.
 */
export default class Server {
    /**
     * Constructs a new instance of the Router class.
     * @param {RouterConfig} config - The configuration object for the router.
     * @returns None
     */
    constructor(config) {
        this.config = config;
        this.routeResolver = new RouteResolver(config);
    }
    /**
     * Returns a callable function that is bound to the `handleServerlessEvent` method of the current object.
     * @returns {CallableFunction} - A callable function that is bound to the `handleServerlessEvent` method.
     */
    getExport() {
        return this.handleServerlessEvent.bind(this);
    }
    /**
     * Handles a serverless event by executing a transaction and resolving the route based on the event.
     * @param {APIGatewayProxyEvent} event - The serverless event object.
     * @param {Context} context - The serverless context object.
     * @returns None
     */
    handleServerlessEvent(event, context) {
        return __awaiter(this, void 0, void 0, function* () {
            // init transaction
            yield new Transaction(event, context, this.config).execute((transaction) => __awaiter(this, void 0, void 0, function* () {
                const request = transaction.request;
                const route = this.routeResolver.resolveRoute(request.getMethod(), request.getPath());
                if (route) {
                    transaction.logger.log('Router accepted route:', route.path);
                    // Validate input
                    if (route.inputSchema) {
                        const validationResp = Validator.validateSchema(request.getBody(), route.inputSchema);
                        if (validationResp && validationResp instanceof Response)
                            return validationResp;
                    }
                    // Validate query
                    if (route.querySchema) {
                        const validationResp = Validator.validateSchema(request.getQueryParams(), route.querySchema);
                        if (validationResp && validationResp instanceof Response)
                            return validationResp;
                    }
                    // parse before validating
                    for (const path of Array.isArray(route.path) ? route.path : [route.path]) {
                        this.parsePathParams(request, path);
                    }
                    // Validate path
                    if (route.pathSchema) {
                        const validationResp = Validator.validateSchema(request.getPathParams(), route.pathSchema);
                        if (validationResp && validationResp instanceof Response)
                            return validationResp;
                    }
                    // Continue to route handler
                    return yield route.handler(transaction);
                }
                //No route found :/
                return new Response(404, { err: 'Route not found!' });
            }));
        });
    }
    /**
     * Parses the path parameters from the request URL based on the given route path.
     * @param {Request<any>} req - The request object.
     * @param {string} routePath - The route path pattern to match against.
     * @returns None
     */
    parsePathParams(req, routePath) {
        const path = req.getPath();
        const regex = pathToRegexp(routePath);
        const result = regex.exec(path);
        if (result)
            req.setFixedPathParams(regex.keys, result);
    }
}
//# sourceMappingURL=Server.js.map