var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import GenericHandlerEvent from './GenericHandlerEvent.js';
import Globals from '../../../Globals.js';
import Utils from '../../../Util/Utils.js';
/**
 * Creates an async function that handles serverless events and sends a response.
 * @param {Server['handleServerlessEvent']} serverlessHandler - The serverless handler function.
 * @returns {Function} - An async function that handles the request and sends a response.
 */
export default (serverlessHandler) => {
    /**
     * Handles an HTTP request by invoking a serverless function and processing the response.
     * @param {Request} request - The HTTP request object.
     * @param {Response} res - The HTTP response object.
     * @returns None
     */
    return (request, res) => __awaiter(void 0, void 0, void 0, function* () {
        const startTime = Date.now();
        try {
            // Generate event with request stuff (http to serverless translation)
            const event = new GenericHandlerEvent(request, serverlessHandler);
            // Invoke
            const invokationResp = yield event.invoke();
            // Respond
            processServerlessResponse(invokationResp, res);
        }
        catch (e) {
            console.error('[Proxy] - Exception during execution!', e);
            console.error(e.stack);
            res.status(Globals.Resp_STATUSCODE_EXCEPTION).json(Object.assign(Object.assign({}, e), { err: Globals.Resp_MSG_EXCEPTION, errCode: Globals.Resp_CODE_EXCEPTION }));
        }
        console.debug(`[Proxy] - Request took ${Date.now() - startTime}ms`);
    });
};
/**
 * Processes the response received from a serverless function invocation and sends the appropriate response back to the client.
 * @param {GenericHandlerEventResponse} invokation - The response object received from the serverless function invocation.
 * @param {Response} res - The response object to send back to the client.
 * @returns The response object to send back to the client.
 */
const processServerlessResponse = (invokation, res) => {
    var _a, _b;
    // translate answer to http layer
    if (invokation && invokation.err) {
        // err came
        return res.json({ err: invokation.err }).status(400);
    }
    else if (!invokation || !invokation.data) {
        // invalid response came
        return res.status(Globals.Resp_STATUSCODE_INVALIDRESP).json({
            err: Globals.Resp_MSG_INVALIDRESP,
            errCode: Globals.Resp_CODE_INVALIDRESP,
        });
    }
    else {
        // Check for headers
        if (invokation.data.headers) {
            for (const hKey of Object.keys(invokation.data.headers)) {
                res.header(hKey, invokation.data.headers[hKey]);
            }
        }
        //
        let response = res.status(invokation.data.statusCode);
        // check for stream
        if ((_b = (_a = invokation.data) === null || _a === void 0 ? void 0 : _a.body) === null || _b === void 0 ? void 0 : _b.pipe) {
            invokation.data.body.pipe(response);
        }
        else {
            response = response.json(Utils.parseObjectNullIfEmpty(invokation.data.body) || {});
        }
        return response;
    }
};
//# sourceMappingURL=GenericHandler.js.map