var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import cuid from 'cuid';
import { parseMultiValueQueryStringParameters, parseQueryStringParameters } from './Utils.js';
import Globals from '../../../Globals.js';
/**
 * Represents a generic handler event for serverless functions.
 */
export default class GenericHandlerEvent {
    /**
     * Constructs a new instance of the class.
     * @param {Request} request - The request object.
     * @param {Server['handleServerlessEvent']} serverlessHandler - The serverless handler function.
     * @returns None
     */
    constructor(request, serverlessHandler) {
        this.request = request;
        this.serverlessHandler = serverlessHandler;
    }
    /**
     * Invokes the handler function asynchronously and returns a promise that resolves to a GenericHandlerEventResponse.
     * @returns {Promise<GenericHandlerEventResponse>} A promise that resolves to a GenericHandlerEventResponse.
     */
    invoke() {
        return __awaiter(this, void 0, void 0, function* () {
            // eslint-disable-next-line no-async-promise-executor
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    // Build event and context
                    const event = this.buildEvent();
                    const context = this.buildContext(event, (err, data) => {
                        resolve({ err, data });
                    });
                    // Invoke
                    yield this.serverlessHandler(event, context);
                }
                catch (e) {
                    reject(e); // forward rejection
                }
            }));
        });
    }
    /**
     * Builds and returns an APIGatewayProxyEvent object based on the current request.
     * @returns {APIGatewayProxyEvent} - The constructed APIGatewayProxyEvent object.
     */
    buildEvent() {
        var _a, _b, _c, _d;
        return {
            body: this.request.body || null, //enforce key
            rawBody: this.request['rawBody'],
            headers: (this.request.headers || {}),
            httpMethod: (_a = this.request.method) === null || _a === void 0 ? void 0 : _a.toUpperCase(),
            isBase64Encoded: false,
            multiValueHeaders: (this.request.headers || {}),
            multiValueQueryStringParameters: parseMultiValueQueryStringParameters(this.request.url),
            path: this.request.path,
            pathParameters: null,
            queryStringParameters: this.request.url ? parseQueryStringParameters(this.request.url) : {},
            requestContext: {
                accountId: process.env.AWS_ACCOUNT_ID || 'undefined',
                apiId: '',
                authorizer: null,
                domainName: undefined,
                domainPrefix: undefined,
                extendedRequestId: cuid(),
                httpMethod: this.request.method ? this.request.method.toUpperCase() : '',
                identity: {
                    accessKey: null,
                    accountId: process.env.AWS_ACCOUNT_ID || null,
                    caller: null,
                    apiKey: null,
                    apiKeyId: null,
                    clientCert: null,
                    cognitoAuthenticationProvider: null,
                    cognitoAuthenticationType: null,
                    cognitoIdentityId: null,
                    cognitoIdentityPoolId: null,
                    principalOrgId: null,
                    sourceIp: ((_b = this.request.headers) === null || _b === void 0 ? void 0 : _b['x-forwarded-for']) ||
                        ((_c = this.request.socket) === null || _c === void 0 ? void 0 : _c.remoteAddress) ||
                        '',
                    user: null,
                    userAgent: ((_d = this.request.headers) === null || _d === void 0 ? void 0 : _d['user-agent']) || null,
                    userArn: null,
                },
                path: this.request.path,
                protocol: 'HTTP/1.1',
                requestId: `${cuid()}-${cuid()}`,
                requestTime: new Date().toISOString(),
                requestTimeEpoch: Date.now(),
                resourceId: '',
                resourcePath: Globals.Listener_HTTP_ProxyRoute,
                stage: process.env.STAGE || '',
            },
            resource: Globals.Listener_HTTP_ProxyRoute,
            stageVariables: null,
        };
    }
    /**
     * Builds and returns a context object for an AWS Lambda function.
     * @param {APIGatewayProxyEvent} event - The event object passed to the Lambda function.
     * @param {(err?: Error | string, data?: any) => void} callback - The callback function to be called when the Lambda function completes.
     * @returns {Context} - The context object for the Lambda function.
     */
    buildContext(event, callback) {
        return {
            awsRequestId: event.requestContext.requestId,
            callbackWaitsForEmptyEventLoop: true,
            getRemainingTimeInMillis: () => {
                return 0;
            },
            done: (err, data) => callback(err, data),
            fail: err => callback(err),
            succeed: res => callback(undefined, res),
            functionName: 'container-function',
            functionVersion: 'LATEST',
            invokedFunctionArn: 'none',
            memoryLimitInMB: '-1',
            logGroupName: 'undefined',
            logStreamName: 'undefined',
        };
    }
}
//# sourceMappingURL=GenericHandlerEvent.js.map