import Server from './../Server.js';
import { RouterConfig } from '../../Router.js';
/**
 * Represents a Proxy class that handles routing and server functionality.
 */
export default class Proxy {
    /**
     * A boolean flag indicating whether the process is currently stopping or not.
     */
    private stopping;
    /**
     * The configuration object for the router.
     */
    private readonly config;
    /**
     * The Express application instance for the server.
     * @readonly
     * @type {express.Express}
     */
    private readonly app;
    /**
     * The handler function for serverless events in the Server class.
     * @param {ServerlessEvent} event - The serverless event object.
     * @returns None
     */
    private readonly serverlessHandler;
    /**
     * Represents a listener for an HTTP server.
     * @private
     * @type {HTTPServer}
     */
    private listener;
    /**
     * Constructs a new instance of the Router class.
     * @param {RouterConfig} config - The configuration object for the router.
     * @param {Server['handleServerlessEvent']} serverlessHandler - The handler function for serverless events.
     * @returns None
     */
    constructor(config: RouterConfig, serverlessHandler: Server['handleServerlessEvent']);
    /**
     * Loads the necessary components and initializes the application.
     * @returns None
     */
    load(): Promise<void>;
    /**
     * Unloads the current module, stopping any active listeners.
     * @param {any} [err] - Optional error object to pass to the stopListeners method.
     * @returns {Promise<void>} - A promise that resolves once the listeners have been stopped.
     */
    unload(err?: any): Promise<void>;
    /**
     * Starts the listeners for the proxy server.
     * @returns {Promise<void>} A promise that resolves when the listeners have started.
     */
    private startListeners;
    /**
     * Stops the listeners and exits the process.
     * @param {any} [err] - Optional error object.
     * @returns {Promise<void>} - A promise that resolves when the listeners are stopped and the process is exited.
     */
    private stopListeners;
    /**
     * Installs the routes for the proxy server.
     * @returns None
     */
    private installRoutes;
}
