var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import fs from 'fs';
import { createServer } from 'http';
import cors from 'cors';
import express from 'express';
import GenericHandler from './GenericHandler.js';
import HealthHandler from './HealthHandler.js';
import Globals from '../../../Globals.js';
import Utils from '../../../Util/Utils.js';
/* Get package.json version from IKApi on ESM */
const { version: appVersion } = JSON.parse(fs.readFileSync('package.json').toString());
/**
 * Represents a Proxy class that handles routing and server functionality.
 */
export default class Proxy {
    /**
     * Constructs a new instance of the Router class.
     * @param {RouterConfig} config - The configuration object for the router.
     * @param {Server['handleServerlessEvent']} serverlessHandler - The handler function for serverless events.
     * @returns None
     */
    constructor(config, serverlessHandler) {
        this.stopping = false;
        this.config = config;
        this.serverlessHandler = serverlessHandler;
        this.app = express();
        /* Opinionated Express configs */
        this.app.use(express.json({
            verify(req, res, buf) {
                req['rawBody'] = buf;
            },
        }));
        // apply cors config
        const corsConfig = this.config.cors || Utils.parseObjectNullIfEmpty(process.env.CORS);
        this.app.use(cors(corsConfig
            ? {
                origin: corsConfig.origin,
                allowedHeaders: corsConfig.headers,
                credentials: !!corsConfig.allowCredentials,
            }
            : {}));
        // //This supposedly fix some 502 codes where nodejs socket would hang during
        // //a request and if behind ALB, it would cause 502 codes. Had experiencied this
        // //and 502 codes reduced dramastically, but still some appearances. Maybe this
        // //is just a palliative work-around for the real issue; TODO: need to investigate
        // //in the future.
        // this.listener.listener.keepAliveTimeout = 120e3
        // this.listener.listener.headersTimeout = 120e3
    }
    /**
     * Loads the necessary components and initializes the application.
     * @returns None
     */
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.startListeners();
            this.installRoutes();
        });
    }
    /**
     * Unloads the current module, stopping any active listeners.
     * @param {any} [err] - Optional error object to pass to the stopListeners method.
     * @returns {Promise<void>} - A promise that resolves once the listeners have been stopped.
     */
    unload(err) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.stopListeners(err);
        });
    }
    /**
     * Starts the listeners for the proxy server.
     * @returns {Promise<void>} A promise that resolves when the listeners have started.
     */
    startListeners() {
        return __awaiter(this, void 0, void 0, function* () {
            // eslint-disable-next-line no-async-promise-executor
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                const port = this.config.port || Globals.Listener_HTTP_DefaultPort;
                console.log(`[Proxy] - [STARTING] - v.${appVersion} - :${port}`);
                // Create Server
                this.listener = createServer(this.app);
                // Set defaults
                this.listener.setTimeout(this.config.timeout || Globals.Listener_HTTP_DefaultTimeout);
                // Call hook if available
                if (this.config.containerSetupHook)
                    yield this.config.containerSetupHook(this.listener, this.app);
                // Start Server
                this.listener.listen(port, () => {
                    console.log(`[Proxy] - [STARTED]`);
                    resolve();
                });
            }));
        });
    }
    /**
     * Stops the listeners and exits the process.
     * @param {any} [err] - Optional error object.
     * @returns {Promise<void>} - A promise that resolves when the listeners are stopped and the process is exited.
     */
    stopListeners(err) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.stopping)
                return;
            this.stopping = true;
            console.debug('[Proxy] - [STOPPING]');
            return new Promise(resolve => {
                this.listener.close(_err => {
                    const err2 = err || _err;
                    if (err2)
                        console.log('[Proxy] - exit output:', err2);
                    console.log('[Proxy] - [STOPPED]');
                    process.exit(err2 ? 1 : 0);
                    resolve(null);
                });
            });
        });
    }
    /**
     * Installs the routes for the proxy server.
     * @returns None
     */
    installRoutes() {
        //Health check route -- This is a bypass route to only check if
        //runtime proxy is working and responding to calls.
        console.log(`[Proxy] - [HEALTH-ROUTE] - ${this.config.healthCheckRoute || Globals.Listener_HTTP_DefaultHealthCheckRoute}`);
        this.app
            .route(this.config.healthCheckRoute || Globals.Listener_HTTP_DefaultHealthCheckRoute)
            .get(HealthHandler);
        //Main route -- We use a wildcard route because is not the job of the runtime and neither
        //the task to deny/constrain routes that invoked this task; all the job is done by the
        //load balancer and we just foward everything we have to the function.
        this.app.route(Globals.Listener_HTTP_ProxyRoute).all(GenericHandler(this.serverlessHandler));
    }
}
//# sourceMappingURL=Proxy.js.map