var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * Represents a class that can be used to create a singleton instance of a class that is created asynchronously.
 */
export default class AsyncSingleton {
    /**
     * Creates a new instance of the AsyncSingleton class.
     * @param factory The factory function that creates the instance.
     * @param checker The function that checks if the instance is valid.
     */
    constructor(factory, checker = () => __awaiter(this, void 0, void 0, function* () { return true; })) {
        this.queue = [];
        this.instantiating = false;
        this.factory = factory;
        this.checker = checker;
    }
    /**
     * Gets the instance of the class.
     * @param r The configuration object for creating the instance.
     */
    instance(r) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.value && (yield this.checker(this.value)))
                return this.value;
            if (this.instantiating) {
                return yield this.awaitOtherInit();
            }
            return yield this.initHere(r);
        });
    }
    /**
     * Gets the current value, may be undefined or invalid.
     */
    getValue() {
        return this.value;
    }
    /**
     * Clears the current value and rejects all pending promises.
     */
    clear() {
        this.value = undefined;
        for (const [, rej] of this.queue) {
            rej();
        }
        this.queue = [];
    }
    initHere(r) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.instantiating = true;
                this.value = yield this.factory(r);
                for (const [res] of this.queue) {
                    res();
                }
                this.queue = [];
                return this.value;
            }
            finally {
                for (const [, rej] of this.queue) {
                    rej();
                }
                this.instantiating = false;
            }
        });
    }
    awaitOtherInit() {
        return __awaiter(this, void 0, void 0, function* () {
            yield new Promise((res, rej) => {
                this.queue.push([res, rej]);
            });
            return this.value;
        });
    }
}
//# sourceMappingURL=AsyncSingleton.js.map