import type {
  Context,
  DynamoDBBatchResponse,
  DynamoDBRecord,
  DynamoDBStreamEvent,
} from 'aws-lambda'

import Transaction, { TransactionConfig } from './Transaction.js'
import Response, { ResponseErrorType } from '../API/Response.js'
import Globals from '../Globals.js'
import Utils from '../Util/Utils.js'

export interface DynamoDBMarshalledRecord extends DynamoDBRecord {
  marshalled: {
    Keys?: object
    OldImage?: object
    NewImage?: object
  }
}

export type DynamoTransactionExecution<ResponseInnerType> = (
  transaction: Transaction<
    null,
    ResponseInnerType | ResponseErrorType,
    DynamoDBBatchResponse | null
  >,
  recordContent: DynamoDBMarshalledRecord
) => Promise<Response<ResponseInnerType | ResponseErrorType> | DynamoDBBatchResponse | null>

export default class DynamoTransaction<ResponseInnerType> {
  private readonly allowFailure: boolean

  private readonly config: TransactionConfig

  private readonly context: Context

  private readonly event: DynamoDBStreamEvent

  constructor(
    event: DynamoDBStreamEvent,
    context: Context,
    config?: TransactionConfig,
    allowFailure?: boolean
  ) {
    this.event = event
    this.context = context
    this.config = config || {}
    this.allowFailure = !!allowFailure
  }

  public async processEvent(
    execution: DynamoTransactionExecution<ResponseInnerType>
  ): Promise<Response<ResponseErrorType | null> | null | DynamoDBBatchResponse> {
    const resp = await this.processRawEvent(execution)
    if (
      !this.allowFailure &&
      resp &&
      resp instanceof Response &&
      !(resp.getCode() >= 200 && resp.getCode() < 300)
    )
      throw new Error(JSON.stringify(resp.getBody() || {}))
    else if (resp) return resp
    return null
  }

  private async processRawEvent(
    execution: DynamoTransactionExecution<ResponseInnerType>
  ): Promise<Response<ResponseErrorType | any> | null | DynamoDBBatchResponse> {
    // safe check for empty events?
    if (this.event.Records && this.event.Records.length > 0) {
      // Regular cleanup of tmp disk to avoid tmp overflow on reused lambdas
      if (!this.config?.skipCleanTmp) await Utils.cleanTemporaryFolder()
      // init transaction for all records
      return await new Transaction<null, any, DynamoDBBatchResponse | null>(
        this.event,
        this.context,
        {
          ...this.config,
          syncReturn: true,
        }
      ).execute(async transaction => {
        // for each available event
        const failureIDs: Array<string> = []
        for (const eventRecordIdx in this.event.Records) {
          const eventRecord = this.event.Records[eventRecordIdx]
          const record = {
            ...eventRecord,
            marshalled: {
              ...(eventRecord.dynamodb?.Keys
                ? { Keys: Utils.ddbUnmarshall(eventRecord.dynamodb?.Keys) }
                : {}),
              ...(eventRecord.dynamodb?.OldImage
                ? { OldImage: Utils.ddbUnmarshall(eventRecord.dynamodb?.OldImage) }
                : {}),
              ...(eventRecord.dynamodb?.NewImage
                ? { NewImage: Utils.ddbUnmarshall(eventRecord.dynamodb?.NewImage) }
                : {}),
            },
          }
          // Call execution with marshalled item
          const resp = await execution(transaction, record)
          // check for failure
          if (
            !resp ||
            (resp instanceof Response && !(resp?.getCode() >= 200 && resp?.getCode() < 300))
          ) {
            // response with failures or fail hard at first
            if (this.allowFailure) failureIDs.push(eventRecord.eventID!)
            else return resp
          }
        }
        // not errored and loop ended - succeeded (might have failures)
        if (this.allowFailure)
          return {
            batchItemFailures: failureIDs.map(id => ({ itemIdentifier: id })),
          }
        return Response.SuccessResponse(null)
      })
    } else
      return Response.BadRequestResponse(
        Globals.ErrorResponseNoRecords,
        Globals.ErrorCode_NoRecords
      )
  }
}
