import { Agent } from 'https'

import { DynamoDBClient } from '@aws-sdk/client-dynamodb'
import { NodeHttpHandler } from '@smithy/node-http-handler'

import { Database } from '../../Database.js'
import type { DbConfig } from '../../types.js'

/**
 * Represents a DynamoDB database connection that extends a generic Database class.
 * @extends Database<any>
 */
export class DynamoDatabase extends Database<any> {
  /**
   * A private static property that holds an instance of the DynamoDBClient class.
   * This property can be used to interact with DynamoDB services.
   */
  private static dynamoProvider = DynamoDBClient
  /**
   * A readonly property representing the DynamoDB client used to interact with DynamoDB.
   */
  public readonly client: DynamoDBClient

  /**
   * Constructor for creating a new instance of a class that interacts with a DynamoDB database.
   * @param {DbConfig<'dynamo'>} config - The configuration object for DynamoDB.
   * @returns None
   */
  constructor(config: DbConfig<'dynamo'>) {
    super(config)
    this.client = this.providerFactory(config)
  }

  /**
   * Override method to handle a transaction asynchronously.
   * @returns A Promise that resolves to null.
   */
  public override async transaction(): Promise<any> {
    return Promise.resolve(null)
  }

  /**
   * Creates a provider for DynamoDB based on the provided configuration.
   * @param {DbConfig<'dynamo'>} config - The configuration object for DynamoDB.
   * @returns A DynamoDB provider instance based on the provided configuration.
   */
  private providerFactory(config: DbConfig<'dynamo'>) {
    return new DynamoDatabase.dynamoProvider({
      region: config.region,
      maxAttempts: config.maxAttempts || 3,
      requestHandler: new NodeHttpHandler({
        connectionTimeout: config.connectionTimeout || 60000,
        socketTimeout: config.connectionTimeout || 60000,
        httpsAgent: new Agent({ keepAlive: false, maxSockets: 50, rejectUnauthorized: true }),
      }),
    })
  }
}
