import Proxy from './container/Proxy.js'
import Server from './Server.js'
import { RouterConfig } from '../Router.js'

/**
 * Represents a server container that extends the Server class and handles serverless events.
 */
export default class ContainerServer extends Server {
  /**
   * The proxy object used for handling requests and responses.
   * @type {Proxy}
   * @protected
   */
  protected readonly proxy: Proxy

  /**
   * Constructs a new instance of the Router class.
   * @param {RouterConfig} config - The configuration object for the router.
   * @returns None
   */
  constructor(config: RouterConfig) {
    super(config)
    this.proxy = new Proxy(config, this.handleServerlessEvent.bind(this))
    this.listenProcessEvents()
  }

  /**
   * Returns a callable function that can be used to export data.
   * @returns {CallableFunction} - A callable function that can be used to export data.
   */
  public getExport(): CallableFunction {
    // start server socket
    this.start()
    // return empty function, we are all done
    return () => {}
  }

  /**
   * Starts the application by loading the proxy.
   * @returns {Promise<void>} - A promise that resolves when the proxy is loaded.
   */
  public async start() {
    await this.proxy.load()
  }

  /**
   * Stops the execution of the program and unloads the proxy.
   * @param {any} [err] - Optional error object to pass to the unload method.
   * @returns {Promise<void>} - A promise that resolves once the proxy is unloaded.
   */
  public async stop(err?: any) {
    await this.proxy.unload(err)
  }

  /**
   * Listens for process events and handles them accordingly.
   * @private
   * @returns None
   */
  private listenProcessEvents() {
    // start process listeners
    process.on('unhandledRejection', this.stop.bind(this)) // listen to exceptions
    process.on('SIGINT', this.stop.bind(this)) // listen on SIGINT signal and gracefully stop the server
  }
}
