import { jest } from '@jest/globals'

import Redis from '../../src/Cache/Redis.js'

let _connectionId = 1

async function simpleRedisTest(config: any, concurrent?: boolean) {
  const RedisMock = jest.fn(() => {
    let opened = false
    const connectionId = _connectionId++
    return {
      connectionId,
      isOpen: () => opened,
      close: () => (opened = false),
      connect: jest.fn(() => {
        opened = true
        return {
          mget: jest.fn(() => {
            return true
          }),
        }
      }),
    } as any
  })
  Redis['ClusterFactory'] = RedisMock
  // Double call is intentional
  const [provider, provider2] = concurrent
    ? await Promise.all([Redis.connection(config), Redis.connection(config)])
    : [await Redis.connection(config), await Redis.connection(config)]

  // client checks
  expect(RedisMock).toHaveBeenNthCalledWith(1, {
    defaults: {
      username: config.username,
      ...(config.password ? { password: config.password } : {}),
      socket: {
        tls: config.enableTLS,
        connectTimeout: 10000,
      },
    },
    rootNodes: [
      {
        url: `redis://${config.username}:${config.username}@${config.hostname}:6379`,
        disableOfflineQueue: true,
      },
    ],
  })
  // Does not have double connection
  expect(provider['connectionId']).toEqual(provider2['connectionId'])
  expect(provider.connect).toHaveBeenCalledTimes(1)
  expect(provider2.connect).toHaveBeenCalledTimes(1)
}

describe('Redis (cluster)', () => {
  beforeEach(async () => {
    // hack to close singleton connection
    if (Redis['singleton'].getValue()) Redis['singleton'].clear()
  })
  test('Simple redis - do not connect twice', async () => {
    await simpleRedisTest({
      hostname: 'redis://localhost',
      username: 'gabe',
      password: 'mypassword',
      enableTLS: true,
      clusterMode: true,
      type: 'redis',
    })
  })
  test('Simple redis (no SSL) - do not connect twice', async () => {
    await simpleRedisTest({
      hostname: 'redis://localhost',
      username: 'gabe',
      password: 'mypassword',
      enableTLS: false,
      clusterMode: true,
      type: 'redis',
    })
  })
  test('Simple redis (passwordless) - do not connect twice', async () => {
    await simpleRedisTest({
      hostname: 'redis://localhost',
      username: 'gabe',
      enableTLS: false,
      clusterMode: true,
      type: 'redis',
    })
  })
  test('Concurrent redis - do not connect twice', async () => {
    await simpleRedisTest(
      {
        hostname: 'redis://localhost',
        username: 'gabe',
        enableTLS: false,
        clusterMode: true,
        type: 'redis',
      },
      true
    )
  })
})
