import { SecretsManagerClient, GetSecretValueCommand } from '@aws-sdk/client-secrets-manager'
import { SSMClient, GetParameterCommand } from '@aws-sdk/client-ssm'
import { mockClient } from 'aws-sdk-client-mock'
import { expect } from 'chai'

import EnvironmentVar, { EnvironmentType } from '../../src/Config/EnvironmentVar.js'

const SSMMock = mockClient(SSMClient)
const SecretsManagerMock = mockClient(SecretsManagerClient)
const prefix = '/abc/test-01/'

function testRemoteEnv(mock, type, command) {
  describe(`Optional ${type} environment`, () => {
    // reset mock
    beforeEach(() => {
      mock.reset()
    })

    test('Does not accept sync resolve', async () => {
      mock.on(command).resolves({
        ...(type == EnvironmentType.PlainRemote
          ? {
              Parameter: {
                Value: undefined,
              },
            }
          : {
              SecretString: undefined,
            }),
      })
      let err = null
      try {
        const env = new EnvironmentVar('abc', type, true, prefix)
        env.syncResolve()
      } catch (e) {
        err = e
      }
      expect(err).is.not.null
      expect(err?.['message']).is.not.null
    })

    test('Optional value with null value', async () => {
      mock.on(command).resolves({
        ...(type == EnvironmentType.PlainRemote
          ? {
              Parameter: {
                Value: undefined,
              },
            }
          : {
              SecretString: undefined,
            }),
      })
      const env = new EnvironmentVar('abc', type, true, prefix)
      const token = await env.resolve()
      expect(token).is.undefined
    })

    test('Optional value with empty response', async () => {
      mock.on(command).resolves({})
      const env = new EnvironmentVar('abc', type, true, prefix)
      const token = await env.resolve()
      expect(token).is.undefined
    })

    test('Optional value with valid response', async () => {
      const value = 'abc'
      mock.on(command).resolves({
        ...(type == EnvironmentType.PlainRemote
          ? {
              Parameter: {
                Value: value,
              },
            }
          : {
              SecretString: value,
            }),
      })
      const env = new EnvironmentVar(value, type, true, prefix)
      const token = await env.resolve()
      expect(token).is.not.undefined
      expect(token).to.be.equals(value)
    })

    test('Optional value rejection', async () => {
      const value = 'abc'
      mock.on(command).rejects(new Error('failed!'))
      const env = new EnvironmentVar(value, type, true, prefix)
      const token = await env.resolve()
      expect(token).is.undefined
    })
  })

  describe(`Required ${type} environment`, () => {
    // reset mock
    beforeEach(() => {
      mock.reset()
    })

    test('Does not accept sync resolve', async () => {
      mock.on(command).resolves({
        ...(type == EnvironmentType.PlainRemote
          ? {
              Parameter: {
                Value: undefined,
              },
            }
          : {
              SecretString: undefined,
            }),
      })
      let err = null
      try {
        const env = new EnvironmentVar('abc', type, undefined, prefix)
        env.syncResolve()
      } catch (e) {
        err = e
      }
      expect(err).is.not.null
      expect(err?.['message']).is.not.null
    })

    test('Required fails when null', async () => {
      mock.on(command).resolves({
        ...(type == EnvironmentType.PlainRemote
          ? {
              Parameter: {
                Value: undefined,
              },
            }
          : {
              SecretString: undefined,
            }),
      })
      let err = null
      try {
        const env = new EnvironmentVar('abc', type, false, prefix)
        await env.resolve()
      } catch (e) {
        err = e
      }
      console.log(err)
      expect(err).is.not.null
      expect(err?.['message']).is.not.null
    })

    test('Required fails when empty response', async () => {
      mock.on(command).resolves({})
      let err = null
      try {
        const env = new EnvironmentVar('abc', type, false, prefix)
        await env.resolve()
      } catch (e) {
        err = e
      }
      expect(err).is.not.null
      expect(err?.['message']).is.not.null
    })

    test('Required value with valid response', async () => {
      const value = 'abc'
      mock.on(command).resolves({
        ...(type == EnvironmentType.PlainRemote
          ? {
              Parameter: {
                Value: value,
              },
            }
          : {
              SecretString: value,
            }),
      })
      const env = new EnvironmentVar(value, type, false, prefix)
      const token = await env.resolve()
      expect(token).is.not.undefined
      expect(token).to.be.equals(value)
    })

    test('Required value rejection', async () => {
      mock.on(command).rejects(new Error('failed!'))
      let err = null
      try {
        const env = new EnvironmentVar('abc', type, false, prefix)
        await env.resolve()
      } catch (e) {
        err = e
      }
      expect(err).is.not.null
      expect(err?.['message']).is.not.null
    })
  })
}

/* Remote envs test */
testRemoteEnv(SSMMock, EnvironmentType.PlainRemote, GetParameterCommand)
testRemoteEnv(SecretsManagerMock, EnvironmentType.SecureRemote, GetSecretValueCommand)

describe('Optional Local environment', () => {
  test('Does accept async resolve', async () => {
    const env = new EnvironmentVar('PATH123', EnvironmentType.Local, true, prefix)
    const v = await env.resolve()
    expect(v).is.undefined
  })

  test('Optional value with null value', async () => {
    const env = new EnvironmentVar('PATH123', EnvironmentType.Local, true, prefix)
    const token = env.syncResolve()
    expect(token).is.undefined
  })

  test('Optional value with valid value', async () => {
    const key = 'PATH'
    const env = new EnvironmentVar(key, EnvironmentType.Local, true, prefix)
    const token = env.syncResolve()
    expect(token).is.not.undefined
    expect(token).to.be.equals(process.env[key])
  })
})

describe('Required local environment', () => {
  test('Does accept async resolve', async () => {
    const key = 'PATH'
    const env = new EnvironmentVar(key, EnvironmentType.Local, false, prefix)
    const v = await env.resolve()
    expect(v).is.not.null
    expect(v).to.be.equals(process.env[key])
  })

  test('Required fails when null', async () => {
    let err = null
    try {
      const env = new EnvironmentVar('PATH123', EnvironmentType.Local, false, prefix)
      env.syncResolve()
    } catch (e) {
      err = e
    }
    console.log(err)
    expect(err).is.not.null
    expect(err?.['message']).is.not.null
  })

  test('Required value with valid value', async () => {
    const key = 'PATH'
    const env = new EnvironmentVar(key, EnvironmentType.Local, false, prefix)
    const v = env.syncResolve()
    expect(v).is.not.null
    expect(v).to.be.equals(process.env[key])
  })
})

export {}
