import { jest } from '@jest/globals'
import { expect as c_expect } from 'chai'

// get console ref and mock before logger import
const consoleProxy = console
const mock = jest.spyOn(consoleProxy, 'log')
// import logger after first ref
import Logger from '../../src/Logger/Logger.js'
// get console reference after logger import
const transactionID = '123-456'

function setContainerFlag(isContainer: boolean) {
  if (isContainer) {
    process.env['HYBRIDLESS_RUNTIME'] = 'true'
  } else {
    process.env['HYBRIDLESS_RUNTIME'] = undefined
  }
}

function fixLogTypePrefix(logType: string) {
  if (logType == 'exception') return 'error'
  if (logType == 'warning') return 'warn'
  if (logType == 'log') return 'info'
  return logType
}

function randomDeepObject(count, endKey) {
  if (count <= 1) return { [endKey]: 'value' }
  const object = {}
  for (let i = 0; i < count; i++) {
    const key = (+new Date() * Math.random()).toString(36).substring(0, 6)
    object[key] = randomDeepObject(Math.floor(Math.random() * (count / 2)), endKey)
  }
  return object
}

function testLogs(isContainer: boolean, provider?: Logger) {
  const type = isContainer ? 'container' : 'serverless'
  const loggerType = !provider ? 'Console' : 'Logger'
  const localProvider = provider || console

  test(`${type} - ${loggerType} Log - Suppress sensitive info`, async () => {
    setContainerFlag(isContainer)
    localProvider.log('my password is 123')
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining((isContainer ? `${transactionID} ` : '') + '[INFO] [Logger.test.ts:')
    )
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining('] my password is 123')
    )
  })

  test(`${type} - ${loggerType} Log - Suppress sensitive info (object)`, async () => {
    setContainerFlag(isContainer)
    const object = { password: '123' }
    localProvider.log('TEST', object)
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining((isContainer ? `${transactionID} ` : '') + '[INFO] [Logger.test.ts:')
    )
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining('] TEST {\n  "password": "**SUPPRESSED_SENSITIVE_DATA** (3 len)"\n}')
    )
    // test if object is not mutate
    c_expect(object.password).to.be.equals('123')
  })

  test(`${type} - ${loggerType} Log - Suppress sensitive info (long-object)`, async () => {
    setContainerFlag(isContainer)
    const object = randomDeepObject(99, 'password')
    localProvider.log('TEST', object)
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining((isContainer ? `${transactionID} ` : '') + '[INFO] [Logger.test.ts:')
    )
    expect(consoleProxy.log).toHaveBeenNthCalledWith(1, expect.stringContaining('] TEST'))
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining('"password": "**SUPPRESSED_SENSITIVE_DATA** (5 len)"')
    )
  })

  test(`${type} - ${loggerType} Log - Suppress sensitive info (object with sensitive string)`, async () => {
    setContainerFlag(isContainer)
    localProvider.log({ object: JSON.stringify({ password: '123' }) })
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining((isContainer ? `${transactionID} ` : '') + '[INFO] [Logger.test.ts:')
    )
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining(
        '] {\n  "object": {\n    "password": "**SUPPRESSED_SENSITIVE_DATA** (3 len)"\n  }\n}'
      )
    )
  })

  test(`${type} - ${loggerType} Log - Suppress sensitive info (object with sensitive number)`, async () => {
    setContainerFlag(isContainer)
    localProvider.log({ object: JSON.stringify({ password: 123 }) })
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining((isContainer ? `${transactionID} ` : '') + '[INFO] [Logger.test.ts:')
    )
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining(
        '] {\n  "object": {\n    "password": "**SUPPRESSED_SENSITIVE_DATA** (3 len)"\n  }\n}'
      )
    )
  })

  test(`${type} - ${loggerType} Log - Suppress sensitive info (null key)`, async () => {
    setContainerFlag(isContainer)
    localProvider.log({ object: JSON.stringify({ password: null }) })
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining((isContainer ? `${transactionID} ` : '') + '[INFO] [Logger.test.ts:')
    )
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining('] {\n  "object": {\n    "password": null\n  }\n}')
    )
  })

  test(`${type} - ${loggerType} Log - Suppress sensitive info (null)`, async () => {
    setContainerFlag(isContainer)
    localProvider.log(null)
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining((isContainer ? `${transactionID} ` : '') + '[INFO] [Logger.test.ts:')
    )
    expect(consoleProxy.log).toHaveBeenNthCalledWith(1, expect.stringContaining('] '))
  })

  test(`${type} - ${loggerType} Log - Suppress sensitive info (array)`, async () => {
    setContainerFlag(isContainer)
    localProvider.log('TEST2', [{ password: '1234' }])
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining((isContainer ? `${transactionID} ` : '') + '[INFO] [Logger.test.ts:')
    )
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining(
        '] TEST2 [\n  {\n    "password": "**SUPPRESSED_SENSITIVE_DATA** (4 len)"\n  }\n]'
      )
    )
  })

  test(`${type} - ${loggerType} Log - Circular reference (Error)`, async () => {
    setContainerFlag(isContainer)
    class SelfRefError extends Error {
      public self: SelfRefError
      constructor() {
        super()
        this.self = this
      }
    }
    localProvider.log(new SelfRefError())
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining((isContainer ? `${transactionID} ` : '') + '[INFO] [Logger.test.ts:')
    )
    expect(consoleProxy.log).toHaveBeenNthCalledWith(
      1,
      expect.stringContaining('tests/Logger/Logger.test.ts:')
    )
  })

  for (const logType of ['log', 'debug', 'info', 'warn', 'warning', 'error', 'exception']) {
    test(`${type} - ${loggerType} ${logType}`, async () => {
      setContainerFlag(isContainer)
      localProvider[logType](logType.toUpperCase())
      // alias
      const logTypePrefix = fixLogTypePrefix(logType)
      expect(consoleProxy.log).toHaveBeenNthCalledWith(
        1,
        expect.stringContaining(
          (isContainer ? `${transactionID} ` : '') +
            `[${logTypePrefix.toUpperCase()}] [Logger.test.ts:`
        )
      )
      expect(consoleProxy.log).toHaveBeenNthCalledWith(
        1,
        expect.stringContaining(`] ${logType.toUpperCase()}`)
      )
    })
  }
}

describe('Logger', () => {
  beforeEach(() => {
    mock.mockClear()
  })
  const provider = new Logger(
    {
      logLevel: 'DEBUG',
      sensitiveFilteringKeywords: true,
    },
    transactionID
  )
  provider.notGlobalLogger()
  // Test serverless logs
  testLogs(false, provider)
  testLogs(false)
  // Test container logs
  testLogs(true, provider)
  testLogs(true)
})

export {}
